@@ -18,18 +18,31 @@
  */
 
 /*
- * $Id: configur.h,v 1.8 2000/11/19 06:21:33 mark Exp $
+ * $Id: configur.h,v 1.42 2004/04/16 11:14:02 mark Exp $
  */
 
 /*
  * For platforms that don't use autoconf, put these here...
  */
-# define HAVE_ASSERT_H        1
-# define HAVE_ERRNO_H         1
-# define HAVE_CTYPE_H         1
-# define HAVE_STDARG_H        1
+#define HAVE_ASSERT_H        1
+#define HAVE_ERRNO_H         1
+#define HAVE_CTYPE_H         1
+#define HAVE_STDARG_H        1
+#define HAVE_SYS_STAT_H      1
 
 #if defined(DOS) && defined(EMX)
+# define HAVE_ALLOCA_H        1
+# define HAVE_FTIME           1
+# define HAVE_MATH_H          1
+# define HAVE_RANDOM          1
+#undef HAVE_RANDOM /* needs bsd support and linking with -lbsd */
+# define HAVE_SIGACTION       1
+# define HAVE_STDIO_H         1
+# define HAVE_SYS_TIME_H      1
+# define HAVE_SYSCONF         1
+# define HAVE_TERMIOS_H       1
+# define HAVE_TIME_H          1
+# define HAVE_FCNTL_H         1
 # define HAVE_GRP_H           1
 # define HAVE_PWD_H           1
 # define HAVE_LIMITS_H        1
@@ -57,6 +70,18 @@
 #endif
 
 #if defined(OS2) && defined(EMX)
+# define HAVE_ALLOCA_H        1
+# define HAVE_FTIME           1
+# define HAVE_MATH_H          1
+# define HAVE_RANDOM          1
+#undef HAVE_RANDOM /* needs bsd support and linking with -lbsd */
+# define HAVE_SIGACTION       1
+# define HAVE_STDIO_H         1
+# define HAVE_SYS_TIME_H      1
+# define HAVE_SYSCONF         1
+# define HAVE_TERMIOS_H       1
+# define HAVE_TIME_H          1
+# define HAVE_FCNTL_H         1
 # define HAVE_GRP_H           1
 # define HAVE_PWD_H           1
 # define HAVE_LIMITS_H        1
@@ -96,21 +121,38 @@
 # define HAVE_MEMMOVE
 # define HAVE_STRERROR
 # define HAVE_VSPRINTF
-# define HAVE_GETPID  
+# define HAVE_GETPID
 # define TIME_WITH_SYS_TIME
 #endif
 
 #if defined(OS2) && defined(__WATCOMC__)
 # define HAVE_MALLOC_H
+# define HAVE_PROCESS_H
 # define HAVE_SETJMP_H
+# define HAVE_STRING_H
 # define HAVE_UNISTD_H
+# define HAVE_STRING_H
+# define HAVE_DIRECT_H
+# define HAVE_PROCESS_H    /* at least OpenWatcom */
+# define HAVE_SYS_SOCKET_H /* at least OpenWatcom */
+# define HAVE_NETINET_IN_H /* at least OpenWatcom */
+# define HAVE_SYS_SELECT_H /* at least OpenWatcom */
+# define HAVE_NETDB_H      /* at least OpenWatcom */
+# define HAVE_ARPA_INET_H  /* at least OpenWatcom */
 # define HAVE__FULLPATH
 # define HAVE_PUTENV
+# define HAVE_DIRECT_H
+# define HAVE_TYPES_H
+# define HAVE_SYS_SOCKET_H
+# define HAVE_NETINET_IN_H
+# define HAVE_SYS_SELECT_H
+# define HAVE_NETDB_H
+# define HAVE_ARPA_INET_H
 # define HAVE_MEMCPY
 # define HAVE_MEMMOVE
 # define HAVE_STRERROR
 # define HAVE_VSPRINTF
-# define HAVE_GETPID  
+# define HAVE_GETPID
 # if defined(DYNAMIC)
 #  define DYNAMIC_OS2
 # endif
@@ -119,9 +161,10 @@
 #if defined(WIN32) && defined(__WATCOMC__)
 # define HAVE_MALLOC_H
 # define HAVE_SETJMP_H
-# define HAVE_SIGNAL_H 
+# define HAVE_SIGNAL_H
 # define HAVE_UNISTD_H
 # define HAVE_FTIME
+# define HAVE_FCNTL_H
 # define HAVE__FULLPATH
 # define HAVE__SPLITPATH2
 /* # define HAVE_FTRUNCATE */
@@ -130,13 +173,14 @@
 # define HAVE_SETENV
 # define HAVE_STRERROR
 # define HAVE_VSPRINTF
-# define HAVE_GETPID  
+# define HAVE_GETPID
 # if defined(DYNAMIC)
 #  define DYNAMIC_WIN32
 # endif
 #endif
 
 #if defined(WIN32) && defined(_MSC_VER)
+# define HAVE_FCNTL_H
 # define HAVE_LIMITS_H
 # define HAVE_MALLOC_H
 # define HAVE_PROCESS_H
@@ -153,12 +197,12 @@
 # define HAVE_MEMMOVE
 # define HAVE_STRERROR
 # define HAVE_VSPRINTF
-# define HAVE_GETPID  
+# define HAVE_GETPID
 # if defined(DYNAMIC)
 #  define DYNAMIC_WIN32
 # endif
 #endif
- 
+
 #if defined(WIN32) && defined(__BORLANDC__)
 # define HAVE_LIMITS_H
 # define HAVE_MALLOC_H
@@ -199,7 +243,7 @@
 # define HAVE_MEMMOVE         1
 # define HAVE_STRERROR        1
 # define HAVE_VSPRINTF        1
-# define HAVE_GETPID  
+# define HAVE_GETPID
 # define TIME_WITH_SYS_TIME   1
 # if defined(DYNAMIC)
 #  define DYNAMIC_WIN32
@@ -223,14 +267,42 @@
 # define HAVE_MEMMOVE         1
 # define HAVE_STRERROR        1
 # define HAVE_VSPRINTF        1
-# define HAVE_GETPID  
+# define HAVE_GETPID
 # define TIME_WITH_SYS_TIME   1
 # if defined(DYNAMIC)
 #  define DYNAMIC_WIN32
 # endif
 #endif
 
+#if defined(WIN32) && defined(__LCC__)
+# define HAVE_FCNTL_H         1
+# define HAVE_DIRECT_H        1
+# define HAVE_LIMITS_H        1
+# define HAVE_MALLOC_H        1
+# define HAVE_PROCESS_H       1
+# define HAVE_SETJMP_H        1
+# define HAVE_SIGNAL_H        1
+# define HAVE_STDLIB_H        1
+# define HAVE_STRING_H        1
+# define HAVE_UNISTD_H        1
+# define HAVE__FULLPATH       1
+# define HAVE_PUTENV          1
+/* # define HAVE_GETTIMEOFDAY    1 */
+/* # define HAVE_FTRUNCATE       1 */
+# define HAVE_MEMCPY          1
+# define HAVE_MEMMOVE         1
+# define HAVE_STRERROR        1
+# define HAVE_VSPRINTF        1
+# define HAVE__SPLITPATH      1
+# define HAVE_GETPID          1
+/* # define TIME_WITH_SYS_TIME   1 */
+# if defined(DYNAMIC)
+#  define DYNAMIC_WIN32
+# endif
+#endif
+
 #if defined(DOS) && defined(DJGPP)
+# define HAVE_FCNTL_H
 # define HAVE_GRP_H
 # define HAVE_PWD_H
 # define HAVE_LIMITS_H
@@ -252,7 +324,7 @@
 # define HAVE_STRERROR
 # define HAVE__TRUENAME
 # define HAVE_VSPRINTF
-# define HAVE_GETPID  
+# define HAVE_GETPID
 # ifndef _MAX_PATH
 #  define _MAX_PATH  PATH_MAX
 # endif
@@ -269,7 +341,7 @@
 # define UNIX
 #endif
 
-#if defined(AMIGA) && defined(GCC)
+#if defined(AMIGA) && ( defined(GCC) || defined(__GNUC__) )
 # define HAVE_GRP_H           1
 # define HAVE_PWD_H           1
 # define HAVE_LIMITS_H        1
@@ -280,6 +352,7 @@
 # define HAVE_STRING_H        1
 # define HAVE_UNISTD_H        1
 # define HAVE_SYS_SOCKET_H    1
+# define HAVE_SYS_FCNTL_H     1
 # define HAVE_NETINET_IN_H    1
 # define HAVE_NETDB_H         1
 # define HAVE_ARPA_INET_H     1
@@ -291,11 +364,138 @@
 # define HAVE_MEMMOVE         1
 # define HAVE_STRERROR        1
 # define HAVE_VSPRINTF        1
-# define HAVE_GETPID  
+# define HAVE_GETPID
 # define TIME_WITH_SYS_TIME   1
+# if !defined( _AMIGA )
+#  define _AMIGA
+# endif
+# if !defined( __AMIGADATE__ )
+#  define __AMIGADATE__ "("__DATE__")"
+# endif
 #endif
 
-#if defined(__vms)
+#if defined(MAC)
+/* # define HAVE_GRP_H           1 */
+/* # define HAVE_PWD_H           1 */
+# define HAVE_LIMITS_H        1
+/* # define HAVE_MALLOC_H        1 */
+# define HAVE_SETJMP_H        1
+# define HAVE_SIGNAL_H        1
+# define HAVE_STDLIB_H        1
+# define HAVE_STRING_H        1
+/* # define HAVE_UNISTD_H        1 */
+/*
+# define HAVE_SYS_SOCKET_H    1
+# define HAVE_NETINET_IN_H    1
+# define HAVE_NETDB_H         1
+# define HAVE_ARPA_INET_H     1
+*/
+/* # define HAVE__FULLPATH        1 */
+/*# define HAVE_SETENV          1 */
+/* # define HAVE_FTIME           1 */
+/* # define HAVE_FTRUNCATE       1 */
+# define HAVE_MEMCPY          1
+# define HAVE_MEMMOVE         1
+# define HAVE_STRERROR        1
+# define HAVE_VSPRINTF        1
+# define HAVE_GETPID
+/* # define TIME_WITH_SYS_TIME   1 */
+#endif
+
+/*
+ * The following are for Epoc32 platform
+ */
+#if defined(__VC32__) && defined(__WINS__)
+# define HAVE_SYS_FCNTL_H
+# define HAVE_LIMITS_H
+# define HAVE_PROCESS_H
+# define HAVE_SETJMP_H
+# define HAVE_SIGNAL_H
+# define HAVE_STDLIB_H
+# define HAVE_STRING_H
+/* # define HAVE__FULLPATH */
+/* # define HAVE__SPLITPATH */
+/* # define HAVE_FTRUNCATE */
+/* # define HAVE_MY_WIN32_SETENV */
+# define HAVE_MEMCPY
+# define HAVE_MEMMOVE
+# define HAVE_STRERROR
+# define HAVE_VSPRINTF
+# define HAVE_GETPID
+# define HAVE_SYS_SOCKET_H
+# define HAVE_NETINET_IN_H
+# define HAVE_NETDB_H
+# define HAVE_ARPA_INET_H
+# define HAVE_UNISTD_H
+# if defined(DYNAMIC)
+#  define DYNAMIC_WIN32
+# endif
+#endif
+
+#if defined(__GCC32__) && defined(__EPOC32__)
+# define HAVE_SYS_FCNTL_H     1
+/* # define HAVE_GRP_H           1 */
+/* # define HAVE_PWD_H           1 */
+# define HAVE_LIMITS_H        1
+/*# define HAVE_MALLOC_H        1 */
+# define HAVE_PROCESS_H       1
+# define HAVE_SETJMP_H        1
+# define HAVE_SIGNAL_H        1
+# define HAVE_STDLIB_H        1
+# define HAVE_STRING_H        1
+# define HAVE_UNISTD_H        1
+/* # define HAVE__FULLPATH        1 */
+/* # define HAVE_PUTENV          1 */
+# define HAVE_GETTIMEOFDAY    1
+/*# define HAVE_FTRUNCATE       1 */
+# define HAVE_MEMCPY          1
+# define HAVE_MEMMOVE         1
+# define HAVE_STRERROR        1
+# define HAVE_VSPRINTF        1
+# define HAVE_GETPID          1
+# define TIME_WITH_SYS_TIME   1
+# define HAVE_SYS_SOCKET_H    1
+# define HAVE_NETINET_IN_H    1
+# define HAVE_NETDB_H         1
+# define HAVE_ARPA_INET_H     1
+# if defined(DYNAMIC)
+#  define DYNAMIC_WIN32
+# endif
+#endif
+
+#if defined(SKYOS)
+# include <skyos/libskyos.h>
+# define HAVE_SYS_FCNTL_H     1
+# define HAVE_GRP_H           1
+# define HAVE_PWD_H           1
+# define HAVE_LIMITS_H        1
+/*# define HAVE_MALLOC_H        1 */
+# define HAVE_PROCESS_H       1
+# define HAVE_SETJMP_H        1
+# define HAVE_SIGNAL_H        1
+# define HAVE_STDLIB_H        1
+# define HAVE_STRING_H        1
+# define HAVE_UNISTD_H        1
+/* # define HAVE__FULLPATH        1 */
+/* # define HAVE_PUTENV          1 */
+# define HAVE_GETTIMEOFDAY    1
+/*# define HAVE_FTRUNCATE       1 */
+# define HAVE_MEMCPY          1
+# define HAVE_MEMMOVE         1
+# define HAVE_STRERROR        1
+# define HAVE_VSPRINTF        1
+# define HAVE_GETPID          1
+/*# define TIME_WITH_SYS_TIME   1*/
+# define HAVE_SYS_SOCKET_H    1
+# define HAVE_NETINET_IN_H    1
+# define HAVE_NETDB_H         1
+# define HAVE_ARPA_INET_H     1
+# if defined(DYNAMIC)
+#  define DYNAMIC_SKYOS
+# endif
+#endif
+
+#if defined(VMS)
 # include "vms.h"
 #endif
 
@@ -344,12 +544,6 @@
 # define DEFAULT_LINEOUTTRUNC 1
 #endif
 
-/*
- * Whether the OPEN() and CLOSE() bifs are available by default
- */
-#define DEFAULT_OPEN_BIF 1
-#define DEFAULT_CLOSE_BIF 1
-
 #define DEFAULT_MAKEBUF_BIF 1
 #define DEFAULT_DROPBUF_BIF 1
 #define DEFAULT_BUFTYPE_BIF 1
@@ -358,24 +552,68 @@
 #define DEFAULT_CACHEEXT 0
 #define DEFAULT_PRUNE_TRACE 1
 
-#define DEFAULT_FIND_BIF 1
-
-#define DEFAULT_EXT_COMMANDS_AS_FUNCS  1
-#define DEFAULT_STDOUT_FOR_STDERR      0
-#define DEFAULT_INTERNAL_QUEUES        0
-#define DEFAULT_TRACE_HTML             0
-#define DEFAULT_FAST_LINES_BIF_DEFAULT 1
-#define DEFAULT_ANSI                   1
-#define DEFAULT_PGB_PATCH1             0  /* pgb */
-
 /*
- * Define the following if your machine has putenv(), unfortunately,
- *   Standard C does not include a way to set environmentvariables, just
- *   to read them. But most machines have the putenv() function.
- * If it is not defined, you will not be able to set the environment
- *   variables throught the VALUE() builtin function.
+ * Indicates that Regina treats external commands as functions by default
  */
-/* #define HAVE_PUTENV */ /* handled in makefile - TBD in autoconf */
+#define DEFAULT_EXT_COMMANDS_AS_FUNCS  1
+/*
+ * Indicates that Regina does not send output that normally goes to stderr, to stdout
+ */
+#define DEFAULT_STDOUT_FOR_STDERR      0
+/*
+ * Indicates that Regina allows external queues (and internal queues) by default
+ */
+#define DEFAULT_INTERNAL_QUEUES        0
+/*
+ * Indicates that Regina does not wrap trace output in HTML code by default
+ */
+#define DEFAULT_TRACE_HTML             0
+/*
+ * Indicates that Regina returns 1 or 0 for when calling LINES BIF without optional 2nd argument by default
+ */
+#define DEFAULT_FAST_LINES_BIF_DEFAULT 1
+/*
+ * Indicates that Regina does not observe strict ANSI functionality by default
+ */
+#define DEFAULT_STRICT_ANSI            0
+/*
+ * Indicates that Regina-specific BUFs are available by default
+ */
+#define DEFAULT_REGINA_BIFS            1
+/*
+ * Indicates that Regina does not do strict white space comparison by default
+ */
+#define DEFAULT_STRICT_WHITE_SPACE_COMPARISONS 0
+/*
+ * Indicates if the ARexx BIFs (OPEN, CLOSE, EOF) use AREXX semantics or not
+ * On Amiga and AROS, they are on by default.
+ */
+#if defined(_AMIGA) || defined(__AROS__)
+# define DEFAULT_AREXX_SEMANTICS        1
+#else
+# define DEFAULT_AREXX_SEMANTICS        0
+#endif
+/*
+ * Indicates that AREXX BIFs are available by default on Amiga and AROS, but
+ * OFF on other platforms
+ */
+#if defined(_AMIGA) || defined(__AROS__)
+# define DEFAULT_AREXX_BIFS             1
+#else
+# define DEFAULT_AREXX_BIFS             0
+#endif
+/*
+ * Indicates if the user want the "broken" semantics of ADDRESS COMMAND
+ */
+#define DEFAULT_BROKEN_ADDRESS_COMMAND 0
+/*
+ * Indicates if the user wants outer parentheses when using CALL subroutine.
+ */
+#define DEFAULT_CALLS_AS_FUNCS          0
+/*
+ * Indicates if the user wants the broken queue naming conventions.
+ */
+#define DEFAULT_QUEUES_301              0
 
 /*
  * The FILE_SEPARATOR is one character, and defines the char that separates
@@ -384,23 +622,30 @@
  * directories, from one another in a PATH environment variable.
  */
 
-#if defined(MSDOS) || defined(__WATCOMC__) || defined(_MSC_VER) || defined(DOS) || defined(OS2)
+#if defined(MSDOS) || ( defined(__WATCOMC__) && !defined(__QNX__) ) || defined(_MSC_VER) || defined(DOS) || defined(OS2) || defined(__WINS__) || defined(__EPOC32__) || defined(__LCC__)
+# define FILE_SEPARATORS     "\\/:"
 # define FILE_SEPARATOR      '\\'
 # define FILE_SEPARATOR_STR  "\\"
 # define PATH_SEPARATOR      ';'
 # define PATH_SEPARATOR_STR  ";"
+#elif defined(VMS)
+# define FILE_SEPARATORS    "]"
+# define FILE_SEPARATOR     ']'
+# define FILE_SEPARATOR_STR "]"
+# define PATH_SEPARATOR     '?'
+# define PATH_SEPARATOR_STR "?"
+#elif defined(MAC)
+# define FILE_SEPARATOR     "]"
+# define FILE_SEPARATOR     ']'
+# define FILE_SEPARATOR_STR "]"
+# define PATH_SEPARATOR     '?'
+# define PATH_SEPARATOR_STR "?"
 #else
-# if defined(VMS)
-#  define FILE_SEPARATOR     ']'
-#  define FILE_SEPARATOR_STR "]"
-#  define PATH_SEPARATOR     '?'
-#  define PATH_SEPARATOR_STR "?"
-# else
-#  define FILE_SEPARATOR     '/'
-#  define FILE_SEPARATOR_STR "/"
-#  define PATH_SEPARATOR     ':'
-#  define PATH_SEPARATOR_STR ":"
-# endif
+# define FILE_SEPARATORS    "/"
+# define FILE_SEPARATOR     '/'
+# define FILE_SEPARATOR_STR "/"
+# define PATH_SEPARATOR     ':'
+# define PATH_SEPARATOR_STR ":"
 #endif
 
 
@@ -408,9 +653,6 @@
 #define FIX_ALL_PROTOS
 
 
-#define RXTOLOW(a) (u_to_l[(unsigned char)(a)])
-
-
 #if defined(HPUX) || defined(__hpux)
 # define YY_CHAR_TYPE unsigned char
 #endif
@@ -510,8 +752,6 @@
 # endif
 #endif
 
-#define OLD_REGINA_FEATURES
-
 /*
  * DLOPEN   - dlopen()/dlsym() under SunOS and SysV4
  * HPSHLOAD - shl_load()/shl_findsym() under HP-UX
@@ -520,6 +760,12 @@
  * WIN32    - LoadLibrary()/GetProcAddress() under Win32
  */
 
+/*
+ * QNX 4.2x does have a ftrunc() equivalent: ltrunc()
+ */
+#if defined(__WATCOMC__) && defined(__QNX__)
+# define HAVE_FTRUNCATE
+#endif
 /*
  * If we don't have ftruncate() then we don't want the default setting
  * of LINEOUT() to be to truncate after each new line written out.
