@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid = "$Id: convert.c,v 1.3 2000/06/26 09:46:40 mark Exp $";
+static char *RCSid = "$Id: convert.c,v 1.9 2004/02/10 10:43:51 mark Exp $";
 #endif
 
 /*
@@ -121,7 +121,6 @@ static char *RCSid = "$Id: convert.c,v 1.3 2000/06/26 09:46:40 mark Exp $";
 #include "rexx.h"
 #include <string.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <assert.h>
 
 
@@ -200,6 +199,8 @@ static streng *pack_hex( const tsd_t *TSD, const streng *string )
    const char *end_ptr=NULL ; /* ptr to end+1 in input hex string */
    char *res_ptr=NULL ;       /* ptr to current char in output string */
    int byte_boundary=0 ;      /* boolean, are we at at byte bounary? */
+   int count;                 /* used to count positions */
+   int last_blank=0;          /* used to report last byte errors */
 
    /*
     * Allow one extra char for padding, ignore that allocated string
@@ -219,8 +220,10 @@ static streng *pack_hex( const tsd_t *TSD, const streng *string )
     * Explicitly check for space at start or end. Illegal space within
     * the hex string is checked for during the loop.
     */
-   if ((ptr<end_ptr) && ((isspace(*ptr)) || (isspace(*(end_ptr-1)))))
-       exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+   if ((ptr<end_ptr) && ((rx_isspace(*ptr)) || (rx_isspace(*(end_ptr-1)))))
+   {
+      exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+   }
 
    /*
     * Find the number of hex digits in the first group of hex digits.
@@ -232,7 +235,7 @@ static streng *pack_hex( const tsd_t *TSD, const streng *string )
     * necessary if the first group of hex digits has an even number of
     * digits, but it is cheaper to do it always that check for it.
     */
-   for (; (ptr<end_ptr) && (isxdigit(*ptr)); ptr++ ) ;
+   for (; (ptr<end_ptr) && (rx_isxdigit(*ptr)); ptr++ ) ;
    byte_boundary = !((ptr-string->value)%2) ;
 
 /* Does this statement do anything useful? (Proabably, things crash if
@@ -245,18 +248,21 @@ static streng *pack_hex( const tsd_t *TSD, const streng *string )
     * Stuff hex digits into the output string, and report error
     * for any other type of data.
     */
-   for (ptr=string->value; ptr<end_ptr; ptr++)
+   for (count=1,ptr=string->value; ptr<end_ptr; ptr++, count++)
    {
-      if (isspace(*ptr))
+      if (rx_isspace(*ptr))
       {
          /*
           * Just make sure that this space occurs at a byte boundary,
           * except from that, ignore it.
           */
+         last_blank = count;
          if (!byte_boundary)
-             exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+         {
+            exiterror( ERR_INVALID_HEX_CONST, 1, count )  ;
+         }
       }
-      else if (isxdigit(*ptr))
+      else if (rx_isxdigit(*ptr))
       {
          /*
           * Stuff it into the output array, either as upper or lower
@@ -275,7 +281,9 @@ static streng *pack_hex( const tsd_t *TSD, const streng *string )
          byte_boundary = !byte_boundary ;
       }
       else
-          exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+      {
+         exiterror( ERR_INVALID_HEX_CONST, 3, (char)*ptr )  ;
+      }
    }
 
    /*
@@ -284,7 +292,9 @@ static streng *pack_hex( const tsd_t *TSD, const streng *string )
     * of hex digits ended at a byte boundary; report error if not.
     */
    if (!byte_boundary)
-       exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+   {
+      exiterror( ERR_INVALID_HEX_CONST, 1, last_blank )  ;
+   }
 
    result->len = res_ptr - result->value ;
    assert( result->len <= result->max ) ;
@@ -314,7 +324,7 @@ static streng *pack_hex( const tsd_t *TSD, const streng *string )
  * will also be returned if 'length' is zero, or if 'string' is the
  * nullstring.
  */
-static streng *numerize( const tsd_t *TSD, const streng *string, int length )
+static streng *numerize( tsd_t *TSD, const streng *string, int length )
 {
    int start=0 ;       /* character to start reading at */
    int sign=0 ;        /* is this to be interpreted as signed? */
@@ -476,8 +486,10 @@ streng *std_b2x( tsd_t *TSD, cparamboxptr parms )
     * it contain leading space, or it is the nullstring. The former is
     * an error, so report it if that is the case.
     */
-   if ((ptr>string->value) && ((first_group==0) || (isspace(*(endptr-1)))))
-       exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+   if ((ptr>string->value) && ((first_group==0) || (rx_isspace(*(endptr-1)))))
+   {
+      exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+   }
 
    /*
     * If the string is a proper bin string, we need one hex digit for
@@ -510,7 +522,7 @@ streng *std_b2x( tsd_t *TSD, cparamboxptr parms )
     */
    for (ptr=string->value; ptr<endptr; ptr++)
    {
-      if (isspace(*ptr))
+      if (rx_isspace(*ptr))
       {
          /*
           * The variable 'cur_bit' is a number containing the relative
@@ -520,7 +532,9 @@ streng *std_b2x( tsd_t *TSD, cparamboxptr parms )
           * better be 0, or else there is space within a nibble.
           */
          if (cur_bit!=0)
-             exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+         {
+            exiterror( ERR_INVALID_HEX_CONST, 2, 1+(ptr-string->value) )  ;
+         }
       }
 
       else if (((*ptr)=='0')||((*ptr)=='1'))
@@ -540,7 +554,9 @@ streng *std_b2x( tsd_t *TSD, cparamboxptr parms )
          }
       }
       else
-          exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+      {
+         exiterror( ERR_INVALID_HEX_CONST, 4, *ptr )  ;
+      }
    }
 
    /*
@@ -574,6 +590,7 @@ streng *std_x2b( tsd_t *TSD, cparamboxptr parms )
    char *res_ptr=NULL ;    /* ptr to contents of 'result' */
    int nibble=0 ;          /* holds a nibble while extracting bin digits */
    int count=0 ;           /* loop control variable */
+   int pos=0 ;             /* position in string for error reporting */
 
    /*
     * Check that we got the parameters that we needed. Then initialize
@@ -597,8 +614,14 @@ streng *std_x2b( tsd_t *TSD, cparamboxptr parms )
     */
    if (end_ptr>ptr)
    {
-      if ((isspace(*ptr)) || (isspace(*(end_ptr-1))))
-          exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+      if ((rx_isspace(*ptr)))
+      {
+         exiterror( ERR_INVALID_HEX_CONST, 1, 1 )  ;
+      }
+      if ((rx_isspace(*(end_ptr-1))))
+      {
+         exiterror( ERR_INVALID_HEX_CONST, 1, (end_ptr-ptr) )  ;
+      }
    }
 
    /*
@@ -607,9 +630,9 @@ streng *std_x2b( tsd_t *TSD, cparamboxptr parms )
     * If anything other than spaces or hex digits are found, report
     * an error.
     */
-   for (; ptr<end_ptr; ptr++)
+   for (pos=1; ptr<end_ptr; ptr++,pos++)
    {
-      if (isspace(*ptr))
+      if (rx_isspace(*ptr))
       {
          /*
           * We have found space in the hex string, eat it up, and keep
@@ -624,10 +647,10 @@ streng *std_x2b( tsd_t *TSD, cparamboxptr parms )
          }
          else if (space_stat==1)
          {
-             exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+            exiterror( ERR_INVALID_HEX_CONST, 1, pos )  ;
          }
       }
-      else if (isxdigit(*ptr))
+      else if (rx_isxdigit(*ptr))
       {
          /*
           * We have found a hex digit, chop it into four parts, and
@@ -650,7 +673,9 @@ streng *std_x2b( tsd_t *TSD, cparamboxptr parms )
            space_stat = ((space_stat==1) ? 2 : 1) ;
       }
       else
-          exiterror( ERR_INVALID_HEX_CONST, 0 )  ;
+      {
+         exiterror( ERR_INVALID_HEX_CONST, 3, *ptr )  ;
+      }
    }
 
    /*
@@ -692,8 +717,13 @@ streng *std_c2x( tsd_t *TSD, cparamboxptr parms )
    return unpack_hex( TSD, parms->value ) ;
 }
 
-
-
+static void check_wholenum( tsd_t *TSD, const char *bif, const streng *arg,
+                            num_descr **num )
+{
+   if ( !myiswnumber( TSD, arg, num,
+                      get_options_flag( TSD->currlevel, EXT_STRICT_ANSI ) ) )
+      exiterror( ERR_INCORRECT_CALL, 12, bif, 1, tmpstr_of( TSD, arg ) );
+}
 
 /* ---------------------------------------------------------------------
  * Converts a whole number into char string. This is just a wrapper
@@ -701,15 +731,28 @@ streng *std_c2x( tsd_t *TSD, cparamboxptr parms )
  */
 streng *std_d2c( tsd_t *TSD, cparamboxptr parms )
 {
-   int length=0 ;  /* the length of the output string */
+   int length;  /* the length of the output string */
+   num_descr *num;
 
-   checkparam(  parms,  1,  2 , "D2C" ) ;
-   if ((parms->next)&&(parms->next->value))
-      length = atozpos( TSD, parms->next->value, "D2C", 2 ) ;
+   checkparam( parms,  1,  2 , "D2C" );
+
+   check_wholenum( TSD, "D2C", parms->value, &num );
+   if ( parms->next && parms->next->value )
+      length = atozpos( TSD, parms->next->value, "D2C", 2 );
    else
-      length = (-1) ;
+   {
+      /*
+       * The strange syntax forces a check for non-negative first arg if the
+       * second doesn't exist.
+       */
+      if ( num->negative )
+         exiterror( ERR_INCORRECT_CALL, 13, "D2C", 1,
+                    tmpstr_of( TSD, parms->value ) );
 
-   return str_binerize( TSD, parms->value, length ) ;
+      length = -1;
+   }
+
+   return str_binerize( TSD, num, length );
 }
 
 
@@ -721,50 +764,63 @@ streng *std_d2c( tsd_t *TSD, cparamboxptr parms )
  */
 streng *std_d2x( tsd_t *TSD, cparamboxptr parms )
 {
-   int length=0 ;         /* holds the requested langth of the result */
-   streng *result=NULL ;  /* the output streng */
-   streng *packed=NULL ;  /* tmp variable, holds the packed string */
+   int length;         /* holds the requested langth of the result */
+   streng *result;     /* the output streng */
+   streng *packed;     /* tmp variable, holds the packed string */
+   num_descr *num;
 
    /*
     * Check the parameters, and set 'length' to the specified length, or
     * to -1 if the second parameter was not specified.
     */
-   checkparam(  parms,  1,  2 , "D2X" ) ;
-   if ((parms->next)&&(parms->next->value))
-      length = atozpos( TSD, parms->next->value, "D2X", 2 ) ;
+   checkparam(  parms,  1,  2 , "D2X" );
+
+   check_wholenum( TSD, "D2C", parms->value, &num );
+   if ( parms->next && parms->next->value )
+      length = atozpos( TSD, parms->next->value, "D2X", 2 );
    else
-      length = (-1) ;
+   {
+      /*
+       * The strange syntax forces a check for non-negative first arg if the
+       * second doesn't exist.
+       */
+      if ( num->negative )
+         exiterror( ERR_INCORRECT_CALL, 13, "D2C", 1,
+                    tmpstr_of( TSD, parms->value ) );
+
+      length = -1;
+   }
 
    /*
     * Convert the whole number into a hex string in a two step operation.
     * First it is converted into a char string, and then that char string
     * is converted into a hexstring.
     */
-   packed = str_binerize( TSD, parms->value, (length==(-1))?(-1):((length+1)/2) ) ;
-   result = unpack_hex( TSD, packed ) ;
-   Free_stringTSD( packed ) ;
+   packed = str_binerize( TSD, num, ( length == -1 ) ? -1 : ( length+1 ) / 2 );
+   result = unpack_hex( TSD, packed );
+   Free_stringTSD( packed );
 
    /*
     * Since we used char string as a temporary format, the hex string
     * will now be padded with one extra zero at the left. If we specified
-    * length, and that length does notmatch the actual length, we must
-    * strip away the first zero in 'result'
+    * length, and that length does not match the actual length, we must
+    * strip away the first zero in 'result'.
     *
     * Here we check for length>0, since we want to catch it if length was
     * specified, but not if it was 0. In the latter case, the string will
     * be the nullstring, and we don't need to do anything anyway.
     */
-   if ((length>0) && (Str_len(result)!=length) && Str_len(result))
+   if ( ( length > 0 ) && ( Str_len( result ) != length ) && Str_len( result ) )
    {
-      assert( Str_len(result) == length+1 ) ;
-      memmove( result->value, &result->value[1], --result->len ) ;
+      assert( Str_len( result ) == length + 1 );
+      memmove( result->value, &result->value[1], --result->len );
    }
 
    /*
     * Just to be safe, check that we did get the nullstring if length
     * was specified to 0
     */
-   assert( (length!=0) || (Str_len(result)==0) ) ;
+   assert( ( length != 0 ) || ( Str_len(result) == 0 ) );
 
    /*
     * If length was not specified, there might be leading zeros in the
@@ -779,16 +835,16 @@ streng *std_d2x( tsd_t *TSD, cparamboxptr parms )
     * occur, but it will be handled, since the result should then be
     * the nullstring.
     */
-   if ((length==(-1)) && (result->value[0]=='0'))
+   if ( ( length == -1 ) && ( result->value[0] == '0' ) )
    {
-      assert( Str_len(result)>1 ) ;
-      assert(( result->value[0]=='0') && (result->value[1]!='0')) ;
+      assert( Str_len( result ) > 1 );
+      assert( ( result->value[0] == '0') && ( result->value[1] != '0' ) );
 
-      memmove( result->value, &result->value[1], --result->len ) ;
+      memmove( result->value, &result->value[1], --result->len );
    }
 
    /*
     * That's it, now we just have to get out of here
     */
-   return result ;
+   return result;
 }
