@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid = "$Id: envir.c,v 1.5 2000/11/19 06:23:20 mark Exp $";
+static char *RCSid = "$Id: envir.c,v 1.25 2004/02/10 10:43:55 mark Exp $";
 #endif
 
 /*
@@ -24,10 +24,6 @@ static char *RCSid = "$Id: envir.c,v 1.5 2000/11/19 06:23:20 mark Exp $";
 #include "rexx.h"
 #include <string.h>
 
-#ifdef HAVE_CTYPE_H
-# include <ctype.h>
-#endif
-
 #if defined(VMS)
 # define fork() vfork()
 # ifdef posix_do_command
@@ -36,37 +32,52 @@ static char *RCSid = "$Id: envir.c,v 1.5 2000/11/19 06:23:20 mark Exp $";
 # define posix_do_command __regina_vms_do_command
 #endif
 
-#if defined(DOS) || defined(WIN32) || defined(OS2) || defined(_AMIGA)
-# ifdef posix_do_command
-#  undef posix_do_command
-# endif
-# define posix_do_command __regina_dos_do_command
-#endif
-
-
 struct envir
 {
-   streng *name ;
+   environment e ;
    int type ;
-   int subtype ;
    struct envir *next, *prev ;
 };
 
-void add_envir( tsd_t *TSD, streng *name, int type, int subtype )
+/* clear_environpart sets initial values of an environpart.
+ * attention: cleanup_environpart() closes open file handles and removes
+ *            temporary files, this function not.
+ */
+static void clear_environpart(environpart *ep)
 {
-   struct envir *ptr=NULL ;
-
-   ptr = MallocTSD( sizeof(struct envir)) ;
-   ptr->name = name ;
-   ptr->type = type ;
-   ptr->subtype = subtype ;
-   ptr->prev = TSD->firstenvir ;
-   ptr->next = NULL ;
-   TSD->firstenvir = ptr ;
-   if (ptr->prev)
-      ptr->prev->next = ptr ;
+   ep->currnum = -1 ;
+   ep->maxnum = -1 ;
+   ep->SameAsOutput = 0;
+   ep->FileRedirected = 0;
+   ep->tempname = NULL ;
+   ep->tmp_queue = NULL ;
+   ep->queue = NULL ;
+   ep->type = 0 ;
+   ep->hdls[2] = ep->hdls[1] = ep->hdls[0] = -1;
 }
 
+void add_envir( tsd_t *TSD, const streng *name, int type, int subtype )
+/* an environment with the same name can be added more than once! */
+{
+   struct envir *ptr;
+
+   ptr = MallocTSD( sizeof( struct envir ) );
+   memset( &ptr->e, 0, sizeof(ptr->e) );
+   clear_environpart( &ptr->e.input );
+   clear_environpart( &ptr->e.output );
+   clear_environpart( &ptr->e.error );
+   ptr->e.input.flags.isinput = 1;
+   ptr->e.error.flags.iserror = 1;
+
+   ptr->e.name = Str_dupTSD( name );
+   ptr->e.subtype = subtype;
+   ptr->type = type;
+   ptr->prev = TSD->firstenvir;
+   ptr->next = NULL;
+   TSD->firstenvir = ptr;
+   if ( ptr->prev )
+      ptr->prev->next = ptr;
+}
 
 #ifdef TRACEMEM
 static void markenvir( const tsd_t *TSD )
@@ -77,7 +88,13 @@ static void markenvir( const tsd_t *TSD )
    for (; eptr; eptr=eptr->prev )
    {
       markmemory( eptr, TRC_ENVIRBOX ) ;
-      markmemory( eptr->name, TRC_ENVIRNAME ) ;
+      markmemory( eptr->e.name, TRC_ENVIRNAME ) ;
+      if (eptr->e.input.name)
+         markmemory( eptr->e.input.name, TRC_ENVIRNAME ) ;
+      if (eptr->e.output.name)
+         markmemory( eptr->e.output.name, TRC_ENVIRNAME ) ;
+      if (eptr->e.error.name)
+         markmemory( eptr->e.error.name, TRC_ENVIRNAME ) ;
    }
 }
 #endif /* TRACEMEM */
@@ -88,7 +105,7 @@ static struct envir *find_envir( const tsd_t *TSD, const streng *name )
    struct envir *ptr=NULL ;
 
    for (ptr=TSD->firstenvir; ptr; ptr=ptr->prev)
-      if (!Str_cmp(ptr->name, name))
+      if (!Str_cmp(ptr->e.name, name))
          return ptr ;
 
    return NULL ;
@@ -106,32 +123,166 @@ int envir_exists( const tsd_t *TSD, const streng *name )
       return 1;
 }
 
+static void del_envirpart( const tsd_t *TSD, environpart *e )
+/* Deletes all allocated parts of the environpart e.
+ * The names are set to NULL afterwards, thus allowing multiple calls.
+ */
+{
+   if ( e->name )
+      Free_stringTSD( e->name ) ;
+   if ( e->base )
+      Free_stringTSD( e->base ) ;
+   if ( e->currname )
+      Free_stringTSD( e->currname ) ;
+   e->name =
+   e->base =
+   e->currname = NULL;
+   /* Temporary files must be deleted under all circumstances! */
+   cleanup_envirpart(TSD, e);
+}
 
 void del_envir( tsd_t *TSD, const streng *name )
+/* Deletes all allocated parts of the environment with the given name and
+ * removes it from the linked list.
+ */
 {
    struct envir *ptr=NULL ;
 
    ptr = find_envir( TSD, name ) ;
-   if (ptr)
-   {
-      if (ptr->prev)
-         ptr->prev->next = ptr->next ;
-      if (ptr->next)
-         ptr->next->prev = ptr->prev ;
-      if (TSD->firstenvir==ptr)
-         TSD->firstenvir = ptr->prev ;
-   }
+   if (!ptr)
+      return;
+
+   if (ptr->prev)
+      ptr->prev->next = ptr->next ;
+   if (ptr->next)
+      ptr->next->prev = ptr->prev ;
+   if (TSD->firstenvir==ptr)
+      TSD->firstenvir = ptr->prev ;
+
+   /* Delete the names in the environment */
+   if ( ptr->e.name )
+      Free_stringTSD( ptr->e.name ) ;
+   del_envirpart( TSD, &ptr->e.input ) ;
+   del_envirpart( TSD, &ptr->e.output ) ;
+   del_envirpart( TSD, &ptr->e.error ) ;
+   FreeTSD( ptr ) ;
 }
 
+static void update_environpart( const tsd_t *TSD, environpart *e,
+                                                             const nodeptr new)
+/* e is the environpart which has to be reset. new is the new part and has
+ * to been valid. new->name may be NULL (for NORMAL behaviour).
+ */
+{
+   del_envirpart( TSD, e ) ;
+
+   if (new->name)
+   {
+      e->name = Str_dupTSD( new->name ) ;
+      e->base = Str_makeTSD( 3*sizeof(int) ) ;
+   }
+
+   e->flags = new->u.of ;
+   clear_environpart(e);
+}
+
+int set_envir( const tsd_t *TSD, const streng *envirname, const nodeptr ios )
+/* This function sets all three IO-redirections INPUT, OUTPUT and ERROR.
+ * ios->p[0] must be the redirection of INPUT,
+ * ios->p[1] must be the redirection of OUTPUT,
+ * ios->p[2] must be the redirection of ERROR
+ * ios==NULL || envirname==NULL --> don't do anything.
+ * ios->p[x]==NULL --> don't touch this redirection.
+ * ios->p[x].name may be NULL (for NORMAL behaviour).
+ *
+ * The types of all four nodes may or may not be X_ADDR_WITH.
+ *
+ * returns 1 on success, 0 if the envirname doesn't exist.
+ */
+{
+   struct envir *e;
+
+   if ( ( envirname == NULL ) || ( ios == NULL ) )
+      return 1;
+
+   if ( ( e = find_envir( TSD, envirname ) ) == NULL )
+      return 0;
+   if (ios->p[0]) update_environpart( TSD, &e->e.input,  ios->p[0] );
+   if (ios->p[1]) update_environpart( TSD, &e->e.output, ios->p[1] );
+   if (ios->p[2]) update_environpart( TSD, &e->e.error,  ios->p[2] );
+   e->e.input.flags.isinput = 1;
+   e->e.error.flags.iserror = 1;
+
+   return 1;
+}
+
+static void dup_environpart( const tsd_t *TSD, environpart *dest,
+                                 const nodeptr prefer, const environpart *src )
+/* Copies src to dest if prefer isn't set. In this case prefer is used.
+ */
+{
+   if (prefer)
+      update_environpart( TSD, dest, prefer ) ;
+   else
+   {
+      if (src->name)
+      {
+         dest->name = Str_dupTSD( src->name ) ;
+         dest->base = Str_makeTSD( 3*sizeof(int) ) ;
+      }
+      dest->flags = src->flags ;
+   }
+   clear_environpart(dest);
+}
+
+static struct envir *dup_envir( tsd_t *TSD, const streng *name, cnodeptr ios )
+/* This functions returns a new instance of the environment with the given
+ * name or NULL if there is not such an environment name.
+ * The current IO-redirection settings are overwritten with the
+ * environment parts given in ios->p[0..2] for INPUT, OUTPUT and ERROR.
+ */
+{
+   struct envir *prev, *new;
+
+   if ( ( prev = find_envir( TSD, name ) ) == NULL )
+      return NULL;
+
+   add_envir( TSD, name, prev->type, prev->e.subtype );
+   new = TSD->firstenvir;
+
+   dup_environpart( TSD, &new->e.input,  ios->p[0], &prev->e.input  );
+   dup_environpart( TSD, &new->e.output, ios->p[1], &prev->e.output );
+   dup_environpart( TSD, &new->e.error,  ios->p[2], &prev->e.error  );
+   new->e.input.flags.isinput = 1;
+   new->e.error.flags.iserror = 1;
+
+   return new;
+}
 
 int init_envir( tsd_t *TSD )
 {
-   add_envir( TSD, Str_creTSD("COMMAND"), ENVIR_SHELL, SUBENVIR_COMMAND ) ;
-   add_envir( TSD, Str_creTSD("SYSTEM"), ENVIR_SHELL, SUBENVIR_SYSTEM ) ;
-   add_envir( TSD, Str_creTSD("OS2ENVIRONMENT"), ENVIR_SHELL, SUBENVIR_SYSTEM ) ;
-   add_envir( TSD, Str_creTSD("ENVIRONMENT"), ENVIR_SHELL, SUBENVIR_SYSTEM ) ;
-   add_envir( TSD, Str_creTSD("CMD"), ENVIR_SHELL, SUBENVIR_COMMAND ) ;
-   add_envir( TSD, Str_creTSD("PATH"), ENVIR_SHELL, SUBENVIR_PATH ) ;
+   static const struct {
+      const char *name ;
+      int         subtype ;
+   } locals[] = {
+      { "COMMAND",        SUBENVIR_PATH    } , /* was SUBENVIR_COMMAND */
+      { "SYSTEM",         SUBENVIR_SYSTEM  } ,
+      { "OS2ENVIRONMENT", SUBENVIR_SYSTEM  } ,
+      { "ENVIRONMENT",    SUBENVIR_SYSTEM  } ,
+      { "CMD",            SUBENVIR_PATH    } , /* was SUBENVIR_COMMAND */
+      { "PATH",           SUBENVIR_PATH    } ,
+      { "REGINA",         SUBENVIR_REXX    } ,
+      { "REXX",           SUBENVIR_REXX    }
+   };
+   unsigned i;
+   streng *tmp;
+
+   for ( i = 0; i < sizeof(locals) / sizeof(locals[0]); i++ )
+   {
+      tmp = Str_creTSD( locals[i].name ) ;
+      add_envir( TSD, tmp, ENVIR_SHELL, locals[i].subtype ) ;
+      Free_stringTSD( tmp ) ;
+   }
 #ifdef TRACEMEM
    regmarker( TSD, markenvir ) ;
 #endif
@@ -192,7 +343,7 @@ static int get_io_flag( tsd_t *TSD, streng *command, streng **rxqueue )
                   /* "|" already checked */
                   for ( i = pos + 1, j = 0; i < length; i++ )
                   {
-                     if ( !isspace(command->value[i] ) )
+                     if ( !rx_isspace(command->value[i] ) )
                         break;
                   }
                   if ( i+7 <= length )
@@ -202,7 +353,7 @@ static int get_io_flag( tsd_t *TSD, streng *command, streng **rxqueue )
                         i += 7;
                         for ( ; i < length; i++ )
                         {
-                           if ( !isspace( command->value[i] ) )
+                           if ( !rx_isspace( command->value[i] ) )
                               break;
                            have_space = 1;
                         }
@@ -247,7 +398,7 @@ static int get_io_flag( tsd_t *TSD, streng *command, streng **rxqueue )
                                qname_start = i;
                                for ( ; i < length; i++ )
                                {
-                                  if ( isspace( command->value[i] ) )
+                                  if ( rx_isspace( command->value[i] ) )
                                   {
                                      have_space = 1;
                                      qname_end = i;
@@ -270,7 +421,7 @@ static int get_io_flag( tsd_t *TSD, streng *command, streng **rxqueue )
                                    */
                                   for ( ; i < length; i++ )
                                   {
-                                     if ( !isspace( command->value[i] ) )
+                                     if ( !rx_isspace( command->value[i] ) )
                                         break;
                                   }
                                   if ( i+6 <= length
@@ -320,21 +471,85 @@ static int get_io_flag( tsd_t *TSD, streng *command, streng **rxqueue )
    return flag ;
 }
 
-
-streng *perform( tsd_t *TSD, const streng *command, const streng *envir, cnodeptr this )
+/*
+ * ANSI 8.2.4 and 8.3.5 and others forces us to set some variables showing
+ * the result and to raise some conditions in case of errors after
+ * processing an external command.
+ */
+void post_process_system_call( tsd_t *TSD, const streng *cmd,
+                               int rc_code, const streng *rc_value,
+                               cnodeptr this )
 {
-   int rc=0, io_flag=0, clearq=0;
-   struct envir *eptr=NULL ;
+   int rs;
+   trap *traps;
+   int type;
+
+   /*
+    * ANSI 8.2.4 etc. forces us to do the following.
+    *
+    */
+
+   if ( !TSD->systeminfo->interactive )
+   {
+      if ( rc_value != NULL )
+         set_reserved_value( TSD, POOL0_RC, Str_dupTSD( rc_value ), 0,
+                             VFLAG_STR );
+      else
+         set_reserved_value( TSD, POOL0_RC, NULL, rc_code, VFLAG_NUM );
+   }
+
+   /* set .RS: -1==Failure, 0=OK, 1=Error */
+   if ( rc_code == 0 )
+      rs = 0;
+   else if ( rc_code < 0 )
+      rs = -1;
+   else
+      rs = 1;
+   set_reserved_value( TSD, POOL0_RS, NULL, rs, VFLAG_NUM );
+
+   if ( rc_code )
+   {
+      traceerror( TSD, this, rc_code );
+      traps = gettraps( TSD, TSD->currlevel );
+      type = ( rc_code > 0 ) ? SIGNAL_ERROR : SIGNAL_FAILURE;
+
+      if ( traps[type].on_off )
+         condition_hook( TSD, type, rc_code, 0, this->lineno, Str_dupTSD( cmd ), NULL );
+   }
+}
+
+streng *perform( tsd_t *TSD, const streng *command, const streng *envir, cnodeptr this, cnodeptr overwrite )
+/* If and only if overwrite is set, a new instance of the environment is
+ * temporarily created and reset to the new IO-redirections.
+ */
+{
+   int rc=0, io_flag=0, clearq=0, tempenvir=0;
+   struct envir *eptr;
    streng *retstr=NULL ;
    streng *rxqueue=NULL;
    streng *cmd=Str_dupTSD(command);
    streng *saved_queue=NULL;
    char *rxq=NULL;
 
-   eptr = find_envir( TSD, envir ) ;
+   if ( overwrite )
+   {
+      /*
+       * fixes bug 653214, overwrite was this->p[1] in former versions, which
+       * was wrong is this wasn't an ADDRESS statement.
+       */
+      if (( eptr = dup_envir( TSD, envir, overwrite ) ) != NULL)
+         tempenvir = 1;
+   }
+   else
+   {
+      eptr = find_envir( TSD, envir ) ;
+   }
 
    if (eptr)
    {
+      if ( TSD->restricted
+      &&   eptr->e.subtype )
+         exiterror( ERR_RESTRICTED, 5 )  ;
       switch (eptr->type)
       {
          case ENVIR_PIPE:
@@ -373,7 +588,7 @@ streng *perform( tsd_t *TSD, const streng *command, const streng *envir, cnodept
                   io_flag -= REDIR_CLEAR;
                }
             }
-            rc = posix_do_command( TSD, cmd, io_flag, eptr->subtype ) ;
+            rc = posix_do_command( TSD, cmd, io_flag, &eptr->e, NULL ) ;
             /*
              * Change the current queue name back
              * to the saved name
@@ -381,7 +596,7 @@ streng *perform( tsd_t *TSD, const streng *command, const streng *envir, cnodept
             if ( get_options_flag( TSD->currlevel, EXT_INTERNAL_QUEUES ) )
             {
                if ( clearq )
-                  drop_buffer( TSD, 0);
+                  drop_buffer( TSD, 0 ) ; /* FIXME: Kill the whole queue??? */
                set_queue( TSD, saved_queue );
                if ( rxqueue != NULL )
                   Free_stringTSD( rxqueue );
@@ -390,36 +605,68 @@ streng *perform( tsd_t *TSD, const streng *command, const streng *envir, cnodept
             break ;
 
          default:
-             exiterror( ERR_INTERPRETER_FAILURE, 0 )  ;
+             exiterror( ERR_INTERPRETER_FAILURE, 1, __FILE__, __LINE__, "" )  ;
       }
    }
    else
    {
-#if 0
-      rc = -3 ;
-      retstr = nullstringptr() ;
-#else
       retstr = SubCom( TSD, cmd, envir, &rc ) ;
-#endif
    }
 
-   setvalue(TSD,&RC_name,Str_dupTSD(retstr)) ;
+   if (tempenvir)
+      del_envir( TSD, envir ) ;
 
-   if (rc && this)
-   {
-      trap *traps ;
-      int type ;
-
-      traceerror( TSD, this, rc ) ;
-      traps = gettraps( TSD, TSD->currlevel ) ;
-      type = (rc>0) ? SIGNAL_ERROR : SIGNAL_FAILURE ;
-
-      if ((type==SIGNAL_FAILURE) && (traps[type].on_off))
-         condition_hook( TSD, type, rc, 0, this->lineno, Str_dupTSD(cmd), NULL ) ;
-      else if (traps[SIGNAL_ERROR].on_off)
-         condition_hook( TSD, SIGNAL_ERROR, rc, 0, this->lineno, Str_dupTSD(cmd), NULL ) ;
-   }
+   post_process_system_call( TSD, cmd, rc, retstr, this );
 
    Free_stringTSD( cmd ) ;
    return retstr ;
 }
+
+/* run_popen is a special implementation of a direct call to posix_do_command.
+ * It redirects input and error to "normal" and fetches the output.
+ * This output has blanks as line-delimiters. All lines are returned
+ * concatenated on success; NULL is returned if an errors occurs.
+ * The global variable "RC" contains the return code of the called process.
+ *
+ * The command is executed in the current environment. If the environment
+ * isn't a standard environment, SYSTEM is used.
+ */
+streng *run_popen( tsd_t *TSD, const streng *command, const streng *envir )
+{
+   int rc;
+   streng *retval;
+   struct envir *ptr=NULL ;
+   Queue *q ;
+
+   for (ptr=TSD->firstenvir; ptr; ptr=ptr->prev)
+   {
+      if ((ptr->type == ENVIR_SHELL) && (Str_cmp(ptr->e.name, envir) == 0))
+         break;
+   }
+   if (ptr == NULL)
+   {
+      retval = Str_creTSD("SYSTEM"); /* temporary misuse */
+      ptr = find_envir(TSD, retval);
+      Free_stringTSD(retval) ;
+   }
+
+   /* Create a new environment with no redirections. */
+   add_envir(TSD, ptr->e.name, ENVIR_SHELL, ptr->e.subtype);
+
+   q = find_free_slot( TSD ) ;
+   q->type = QisTemp;
+   rc = posix_do_command(TSD, command, REDIR_OUTSTRING, TSD->firstenvir, q ) ;
+   retval = stack_to_line( TSD, q ) ;
+
+   /* restore the previous environment and delete the temporary one */
+   del_envir(TSD, ptr->e.name);
+
+   set_reserved_value( TSD, POOL0_RC, NULL, rc, VFLAG_NUM );
+
+   if (rc >= 0)
+      return(retval);
+
+   /* rc shows an error while calling: */
+   Free_stringTSD(retval);
+   return(NULL);
+}
