@@ -10,7 +10,7 @@
 
 #include "rexxsaa.h"
 
-#define STARTUPMSG "       +++ Interactive trace. TRACE OFF to end debug, ENTER to continue. +++"
+#define STARTUPMSG "       +++ Interactive trace.  \"Trace Off\" to end debug. ENTER to continue."
 
 #ifdef _MSC_VER
 /* This picky compiler claims about unused formal parameters.
@@ -149,6 +149,21 @@ static void trace( void )
    RexxDeregisterExit( "Foo", NULL ) ;
 }
 
+static void assign_new_rxstring( PRXSTRING dest, const char *source )
+{
+   int len;
+   char *buf;
+
+   len = strlen( source );
+   if ( ( buf = RexxAllocateMemory( len + 1 ) ) == NULL )
+   {
+      fprintf( stderr, "RexxAllocateMemory returns NULL.\n" );
+      exit( 1 );
+   }
+   strcpy( buf, source );
+   MAKERXSTRING( *dest, buf, len );
+
+}
 
 LONG APIENTRY intertrc_exit( LONG ExNum, LONG Subfun, PEXIT PBlock )
 {
@@ -197,8 +212,7 @@ LONG APIENTRY intertrc_exit( LONG ExNum, LONG Subfun, PEXIT PBlock )
        case 9:
           iverify( "Subfunction", Subfun, RXSIODTR ) ;
           psiodtr = (RXSIODTR_PARM *)PBlock;
-          psiodtr->rxsiodtr_retc.strptr = data[cnt] ;
-          psiodtr->rxsiodtr_retc.strlength = strlen(data[cnt]) ;
+          assign_new_rxstring( &psiodtr->rxsiodtr_retc, data[cnt] );
           break ;
 
        default:
@@ -265,8 +279,7 @@ LONG APIENTRY pull_exit( LONG ExNum, LONG Subfun, PEXIT PBlock )
        case 4:
           iverify( "Subfunction", Subfun, RXSIOTRD ) ;
           psiotrd = (RXSIOTRD_PARM *)PBlock;
-          strcpy(psiotrd->rxsiotrd_retc.strptr,data[cnt]) ;
-          psiotrd->rxsiotrd_retc.strlength = strlen(data[cnt]) ;
+          assign_new_rxstring( &psiotrd->rxsiotrd_retc, data[cnt] );
           break ;
 
        default:
@@ -302,6 +315,71 @@ static void pull( void )
 }
 
 
+LONG APIENTRY env_exit( LONG ExNum, LONG Subfun, PEXIT PBlock )
+{
+   static int cnt=0 ;
+   static char data[10];
+   RXENVGET_PARM *penvget;
+   RXENVSET_PARM *penvset;
+
+   iverify( "Exitfunction", ExNum, RXENV ) ;
+   switch (++cnt)
+   {
+       case 1:
+          iverify( "Subfunction", Subfun, RXENVGET ) ;
+          penvget = (RXENVGET_PARM *)PBlock;
+          sverify( "Envget name", &(penvget->rxenv_name),"FRED");
+          assign_new_rxstring( &penvget->rxenv_value, "initial" );
+          break ;
+
+       case 2:
+          iverify( "Subfunction", Subfun, RXENVSET ) ;
+          penvset = (RXENVSET_PARM *)PBlock;
+          sverify( "Envset name", &(penvset->rxenv_name),"FRED");
+          sverify( "Envset value", &(penvset->rxenv_value),"junk");
+          strcpy( data, "mess" );
+          break ;
+
+       case 3:
+          iverify( "Subfunction", Subfun, RXENVGET ) ;
+          penvget = (RXENVGET_PARM *)PBlock;
+          sverify( "Envget name", &(penvget->rxenv_name),"FRED");
+          assign_new_rxstring( &penvget->rxenv_value, data );
+          break ;
+
+       default:
+          fprintf( stderr, "Out of order msg subfunc=%ld\n", Subfun) ;
+          return RXEXIT_NOT_HANDLED ;
+   }
+   return RXEXIT_HANDLED ;
+}
+
+static void env( void )
+{
+   RXSTRING Instore[2] ;
+   RXSYSEXIT Exits[2] ;
+   int rc ;
+
+   message( "env") ;
+
+   RexxRegisterExitExe( "Foo", (PFN)env_exit, NULL ) ;
+
+   Exits[0].sysexit_name = "Foo" ;
+   Exits[0].sysexit_code = RXENV ;
+   Exits[1].sysexit_code = RXENDLST ;
+
+   Instore[0].strptr =
+     "call value 'FRED', 'junk', 'ENVIRONMENT'; if value('FRED',,'ENVIRONMENT') <> 'mess' then say 'setenv/getenv failed'" ;
+   Instore[0].strlength = strlen( Instore[0].strptr ) ;
+   Instore[1].strptr = NULL ;
+
+   rc = RexxStart( 0, NULL, "Testing", Instore, "Foo", RXCOMMAND,
+                   Exits, NULL, NULL ) ;
+
+   RexxDeregisterExit( "Foo", NULL ) ;
+}
+
+
 
 
 LONG APIENTRY query_init_exit( LONG ExNum, LONG Subfun, PEXIT PBlock )
@@ -343,21 +421,23 @@ LONG APIENTRY query_term_exit( LONG ExNum, LONG Subfun, PEXIT PBlock )
    iverify( "Query Term exit", rc, RXQUEUE_OK ) ;
    iverify( "Query Term count", count, 2L ) ;
 
+   data.strptr = NULL;
    rc = RexxPullQueue( "FRED", &data, &timestamp, RXQUEUE_NOWAIT );
    iverify( "Query Term exit", rc, RXQUEUE_OK ) ;
    sverify( "Query Term data", &data, "2line" ) ;
    RexxFreeMemory( data.strptr ) ;
 
+   data.strptr = NULL;
    rc = RexxPullQueue( "FRED", &data, &timestamp, RXQUEUE_NOWAIT );
    iverify( "Query Term exit", rc, RXQUEUE_OK ) ;
    sverify( "Query Term data", &data, "1line" ) ;
    RexxFreeMemory( data.strptr ) ;
 
    rc = RexxDeleteQueue( "FRED" );
-   iverify( "Query Init exit", rc, RXQUEUE_OK ) ;
+   iverify( "Query Term exit", rc, RXQUEUE_OK ) ;
 
    rc = RexxDeleteQueue( "TEST1" );
-   iverify( "Query Init exit", rc, RXQUEUE_OK ) ;
+   iverify( "Query Term exit", rc, RXQUEUE_OK ) ;
 
    return RXEXIT_HANDLED ;
 }
@@ -408,7 +488,7 @@ LONG APIENTRY it_term_exit( LONG ExNum, LONG Subfun, PEXIT PBlock )
    name = "bar" ;
    shv.shvname.strptr = name ;
    shv.shvnamelen = shv.shvname.strlength = strlen(name) ;
-   
+
    /* The value is the same as above (value) and therefore valid */
 #if 0
     shv.shvvalue.strptr = value ;
@@ -716,7 +796,7 @@ LONG APIENTRY source_exit( LONG code, LONG subcode, PEXIT ptr )
    sverify( "Parm2b", &(Req[4].shvvalue), NULL ) ;
 
    iverify( "QueName", Req[5].shvret, RXSHV_OK ) ;
-   sverify( "QueName", &(Req[5].shvvalue), "default" ) ;
+   sverify( "QueName", &(Req[5].shvvalue), "SESSION" ) ;
 
    return RXEXIT_HANDLED ;
 }
@@ -1121,7 +1201,7 @@ static void qtest( void )
    Exits[1].sysexit_code = RXTER ;
    Exits[2].sysexit_code = RXENDLST ;
    RexxRegisterExitExe( "query_init", (PFN)query_init_exit, NULL ) ;
-   RexxRegisterExitExe( "query_term", (PFN)query_init_exit, NULL ) ;
+   RexxRegisterExitExe( "query_term", (PFN)query_term_exit, NULL ) ;
 
    Instore[0].strptr = "call rxqueue 'Set', 'test1';"
                        "if queued() \\= 2 then say 'error:line 2';"
@@ -1130,6 +1210,7 @@ static void qtest( void )
                        "parse pull line;"
                        "if line \\= 'line2' then say 'error:line 5';"
                        "if rxqueue('Create','FRED') \\= 'FRED' then say 'error:line 6';"
+                       "if rxqueue('Set','FRED') \\= 'TEST1' then say 'error:line 7';"
                        "push '1line';push '2line';" ;
    Instore[0].strlength = strlen( Instore[0].strptr ) ;
    Instore[1].strptr = NULL ;
@@ -1149,6 +1230,7 @@ static void (*(routines[]))(void) = {
     trace,
     intertrc,
     pull,
+    env,
     init_term,
     vars,
     source,
@@ -1173,6 +1255,15 @@ static RexxSubcomHandler *(scfuncs[]) = {
    NULL
 } ;
 
+char char82( void )
+{
+   return (char) 0x82;
+}
+
+char char0( void )
+{
+   return '\0';
+}
 
 int main( int argc, char *argv[] )
 {
@@ -1193,7 +1284,7 @@ int main( int argc, char *argv[] )
    version.strptr = NULL;
    /* This might not work if we check another Rexx: */
    versioncode = ReginaVersion(&version);
-   printf("Regina's version is %lu.%02lu",
+   printf("Regina's version is %lu.%lu",
           versioncode >> 8,
           versioncode & 0xFF);
    if (version.strptr)
@@ -1203,6 +1294,9 @@ int main( int argc, char *argv[] )
    }
    printf("\n");
 
+   if ( char82() < char0() )
+      printf( "WARNING: Current compiler uses `signed char' as default!\n" );
+
    for( fptr=routines; *fptr; fptr++ )
       (*fptr)() ;
 
@@ -1217,8 +1311,7 @@ int main( int argc, char *argv[] )
 #if !defined(NO_EXTERNAL_QUEUES)
    qtest();
 #endif
+   ReginaCleanup();
    printf( "\n" ) ;
    return 0 ;
-
 }
-
