@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid = "$Id: expr.c,v 1.3 2000/11/19 06:24:48 mark Exp $";
+static char *RCSid = "$Id: expr.c,v 1.25 2004/02/10 10:43:57 mark Exp $";
 #endif
 
 /*
@@ -24,7 +24,6 @@ static char *RCSid = "$Id: expr.c,v 1.3 2000/11/19 06:24:48 mark Exp $";
 #include "rexx.h"
 
 #include <string.h>
-#include <ctype.h>
 #include <assert.h>
 
 #define TRACEVALUE(a,b) if (TSD->trace_stat=='I') tracevalue(TSD,a,b)
@@ -39,12 +38,23 @@ static void mark_in_expr( const tsd_t *TSD )
 }
 #endif
 
+/*
+ * COMP_IGNORE returns 1 if c shall be ignored on a non-strict comparison
+ * between non-numbers. Previously, this was equal to isspace(c). This was
+ * wrong according to ANSI, section 7.4.7. This fixes bug 594674.
+ */
+#define ANSI_COMP_IGNORE(c) ( (c) == ' ' )
+#define REGINA_COMP_IGNORE(c) ( rx_isspace(c) )
+
+#define FREE_TMP_STRING(str) if ( str )               \
+                                Free_stringTSD( str )
 
 int init_expr( tsd_t *TSD )
 {
 #ifdef TRACEMEM
    regmarker( TSD, mark_in_expr ) ;
 #endif
+   TSD = TSD; /* keep compiler happy */
    return(1);
 }
 
@@ -59,7 +69,9 @@ static num_descr *copy_num( const tsd_t *TSD, const num_descr *input )
    new->max = (input->max < 1) ? 1 : input->max ;
    new->exp = input->exp ;
    new->num = MallocTSD( new->max ) ;
+   new->used_digits = input->used_digits;
    memcpy( new->num, input->num, new->size ) ;
+   TSD = TSD; /* keep compiler happy */
    return new ;
 }
 
@@ -122,18 +134,23 @@ static num_descr *bool_to_num( const tsd_t *TSD, int input )
    num->negative = 0 ;
    num->exp = 1 ;
    num->num[0] = (char) ((input) ? '1' : '0') ;
+   num->used_digits = TSD->currlevel->currnumsize;
    return num ;
 }
 
+/*
+ * calcul evaluates a numeric expression. this is the current evaluation tree.
+ * kill? return value?
+ * Note: This is one of the most time-consuming routines. Be careful.
+ */
 num_descr *calcul( tsd_t *TSD, nodeptr this, num_descr **kill )
 {
-   num_descr *numthr=NULL, *numone=NULL, *numtwo=NULL ;
+   num_descr *numthr, *numone, *numtwo ;
    num_descr *ntmp1=NULL, *ntmp2=NULL ;
-   int power=0 ;
-   num_descr *nptr=NULL ;
-   streng *sptr=NULL ;
+   num_descr *nptr;
+   streng *sptr;
 
-   switch (/*(unsigned char)*/(this->type))
+   switch ( this->type )
    {
       case 0:
       case 255:
@@ -163,7 +180,7 @@ do_an_add:
          else
             numthr = copy_num( TSD, numtwo ) ;
 
-         string_add( TSD, numone, numtwo, numthr ) ;
+         string_add( TSD, numone, numtwo, numthr, this->p[0], this->p[1] ) ;
          break ;
 
       case X_MULT:
@@ -182,7 +199,7 @@ do_an_add:
          else
             numthr = copy_num( TSD, numtwo ) ;
 
-         string_mul( TSD, numone, numtwo, numthr ) ;
+         string_mul( TSD, numone, numtwo, numthr, this->p[0], this->p[1] );
          break ;
 
       case X_DEVIDE:
@@ -196,28 +213,20 @@ do_an_add:
          numthr = copy_num( TSD, numtwo ) ;
          string_div( TSD, numone, numtwo, numthr, NULL,
             ((this->type==X_DEVIDE) ? DIVTYPE_NORMAL :
-            ((this->type==X_MODULUS) ? DIVTYPE_REMINDER : DIVTYPE_INTEGER))) ;
+            ((this->type==X_MODULUS) ? DIVTYPE_REMAINDER : DIVTYPE_INTEGER)),
+            this->p[0], this->p[1] );
          break ;
 
       case X_EXP:
          numone = calcul( TSD, this->p[0], &ntmp1 ) ;
-         power = descr_to_int( calcul( TSD, this->p[1], &ntmp2 )) ;
-         if (ntmp2)
-         {
-            numtwo = ntmp2 ;
-/*            ntmp2 = NULL ; */
-         }
-         else
-            ntmp2 = numtwo = copy_num( TSD, numone ) ;
-
-
+         numtwo = ntmp2 = calcul( TSD, this->p[1], NULL ) ;
          numthr = copy_num( TSD, numone ) ;
-         string_pow( TSD, numone, power, numtwo, numthr ) ;
+         string_pow( TSD, numone, numtwo, numthr, this->p[0], this->p[1] ) ;
          break ;
 
       case X_STRING:
       case X_CON_SYMBOL:
-         if (!this->u.number)
+         if ( !this->u.number )
             this->u.number = get_a_descr( TSD, this->name ) ;
 
          if (TSD->trace_stat=='I')
@@ -249,7 +258,7 @@ do_an_add:
          if (kill)
             *kill = NULL ;
 
-         nptr = fix_compoundnum( TSD, this, NULL ) ;
+         nptr = fix_compoundnum( TSD, this, NULL, NULL );
          if (!nptr)
              exiterror( ERR_BAD_ARITHMETIC, 0 )  ;
 
@@ -283,10 +292,8 @@ do_an_add:
       case X_CONCAT:
       case X_SPACE:
       {
-         streng *stmp ;
-         numthr = str_to_num( TSD, stmp=evaluate( TSD, this, &sptr )) ;
-         if (sptr)
-            Free_stringTSD( sptr ) ;
+         numthr = str_to_num( TSD, evaluate( TSD, this, &sptr ) );
+         FREE_TMP_STRING( sptr );
          if (kill)
             *kill = numthr ;
          return numthr ;
@@ -327,7 +334,7 @@ do_an_add:
          return numthr ;
 
       default:
-          exiterror( ERR_INTERPRETER_FAILURE, 0 )  ;
+         exiterror( ERR_INTERPRETER_FAILURE, 1, __FILE__, __LINE__, "" )  ;
          return NULL ;
    }
 
@@ -352,17 +359,94 @@ do_an_add:
    return numthr ;
 }
 
+static void strip_whitespace( tsd_t *TSD, unsigned char **s1,
+                              unsigned char **e1, unsigned char **s2,
+                              unsigned char **e2 )
+{
+   if ( get_options_flag( TSD->currlevel, EXT_STRICT_ANSI )
+   ||   get_options_flag( TSD->currlevel, EXT_STRICT_WHITE_SPACE_COMPARISONS ) )
+   {
+      /*
+       * ANSI 7.4.7 behaviour; non-strict comparisons
+       * removed ONLY spaces, so single TAB not equal to single SPACE
+       */
+      /*
+       * Strip leading spaces - ignored in comparison
+       */
+      for( ; ( *s1 < *e1 ) && ANSI_COMP_IGNORE( **s1 ); (*s1)++ )
+      {
+      }
+      for( ; ( *s2 < *e2 ) && ANSI_COMP_IGNORE( **s2 ); (*s2)++ )
+      {
+      }
+      for ( ; ( *s1 < *e1 ) && ( *s2 < *e2 ) && ( **s1 == **s2 ); (*s1)++, (*s2)++ )
+      {
+      }
+      /*
+       * Strip trailing spaces - ignored in comparison
+       */
+      for ( ; ( *e1 > *s1 ) && ANSI_COMP_IGNORE( *( *e1 - 1 ) ); (*e1)-- )
+      {
+      }
+      for ( ; ( *e2 > *s2 ) && ANSI_COMP_IGNORE( *( *e2 - 1 ) ); (*e2)-- )
+      {
+      }
+   }
+   else
+   {
+      /*
+       * Original Regina behaviour; non-strict comparisons
+       * removed ALL white space, so single TAB equalled single SPACE
+       */
+      /*
+       * Strip leading white space - ignored in comparison
+       */
+      for( ; ( *s1 < *e1 ) && REGINA_COMP_IGNORE( **s1 ); (*s1)++ )
+      {
+      }
+      for( ; ( *s2 < *e2 ) && REGINA_COMP_IGNORE( **s2 ); (*s2)++ )
+      {
+      }
+      for ( ; ( *s1 < *e1 ) && ( *s2 < *e2 ) && ( **s1 == **s2 ); (*s1)++, (*s2)++ )
+      {
+      }
+      /*
+       * Strip trailing white space - ignored in comparison
+       */
+      for ( ; ( *e1 > *s1 ) && REGINA_COMP_IGNORE( *( *e1 - 1 ) ); (*e1)-- )
+      {
+      }
+      for ( ; ( *e2 > *s2 ) && REGINA_COMP_IGNORE( *( *e2 - 1 ) ); (*e2)-- )
+      {
+      }
+   }
+}
 
 
+/*
+ * evaluate evaluates an expression. The nodeptr "this" must point to an
+ * expression part. The return value is the value of the expression.
+ * For a proper cleanup the caller probably has to delete a the returned
+ * value. For this purpose, the caller may set "kill" to non-NULL.
+ * *kill is set to NULL, if the returned value is a const value and must
+ * not be freed. *kill is set to a temporary value which has to be deleted
+ * after the use of the returned value.
+ * The caller may omit kill, this forces evaluate to create a freshly allocated
+ * return value.
+ */
 streng *evaluate( tsd_t *TSD, nodeptr this, streng **kill )
 {
-   streng *strone=NULL, *strtwo=NULL, *strthr=NULL ;
-   streng *stmp1, *stmp2=NULL, *sptr=NULL ;
-   streng *stmp=NULL ;
-   const streng *cstmp=NULL ;
-   num_descr *ntmp=NULL ;
+#define RETURN_NEW(val) if ( kill )     \
+                           *kill = val; \
+                        return val;
+   streng *strone,*strtwo,*strthr;
+   streng *stmp1,*stmp2;
+   const streng *cstmp;
+   num_descr *ntmp;
 
-   switch (/*(unsigned char)*/(this->type))
+   if ( kill )
+      *kill = NULL;
+   switch ( this->type )
    {
       case 0:
       case 255:
@@ -375,101 +459,92 @@ streng *evaluate( tsd_t *TSD, nodeptr this, streng **kill )
       case X_EXP:
       case X_U_MINUS:
       case X_U_PLUSS:
-         stmp = num_to_str( TSD, calcul( TSD, this, &ntmp )) ;
-         if (ntmp)
+         ntmp = NULL;
+         stmp1 = num_to_str( TSD, calcul( TSD, this, &ntmp ) );
+         if ( ntmp )
          {
-            FreeTSD( ntmp->num ) ;
-            FreeTSD( ntmp ) ;
+            FreeTSD( ntmp->num );
+            FreeTSD( ntmp );
          }
-         if (kill)
-            *kill = stmp ;
-
-         return stmp ;
+         RETURN_NEW( stmp1 );
 
       case X_NULL:
          return NULL ;
 
       case X_STRING:
       case X_CON_SYMBOL:
-         cstmp = this->name ;
-         if (TSD->trace_stat=='I')
-            tracevalue( TSD, cstmp, 'L' ) ;
-
-         goto cnsnt ;
+         cstmp = this->name;
+         if ( TSD->trace_stat == 'I' )
+            tracevalue( TSD, cstmp, 'L' );
+         if ( kill )
+            return (streng *) cstmp; /* and *kill is set to NULL above */
+         stmp1 = Str_dupTSD( cstmp );
+         RETURN_NEW( stmp1 );
 
       case X_HEAD_SYMBOL:
-         /* always duplicate, since stmp might point to tmp area */
-         stmp = Str_dupTSD( fix_compound(TSD, this, NULL)) ;
-         if (kill)
-            *kill = stmp ;
-         return stmp ;
+         /* always duplicate, since stmp1 might point to tmp area */
+         stmp1 = Str_dupTSD( fix_compound( TSD, this, NULL ) );
+         RETURN_NEW( stmp1 );
 
       case X_STEM_SYMBOL:
       case X_SIM_SYMBOL:
          cstmp = shortcut(TSD,this) ;
-cnsnt:   if (kill)
-         {
-            *kill = NULL ;
-            /* FIXME, FGC: I hate the following. The complete code
-             * should be fixed here!
-             */
-            stmp = (streng *) cstmp;
-         }
-         else
-            stmp = Str_dupTSD( cstmp ) ;
-         return stmp ;
+         if ( kill )
+            return (streng *) cstmp; /* and *kill is set to NULL above */
+         stmp1 = Str_dupTSD( cstmp );
+         RETURN_NEW( stmp1 );
 
       case X_IN_FUNC:
       {
-         nodeptr entry ;
-         if ((entry=getlabel(TSD,this->name))!=NULL)
+         nodeptr entry;
+
+         if ( ( entry = getlabel( TSD, this->name ) ) != NULL )
          {
-            this->type = X_IS_INTERNAL ;
-            this->u.node = entry ;
+            if ( entry->u.trace_only )
+               exiterror( ERR_UNEXISTENT_LABEL, 3, tmpstr_of( TSD, this->name ) );
+            this->type = X_IS_INTERNAL;
+            this->u.node = entry;
          }
          else
-            this->u.node = NULL ;
+            this->u.node = NULL;
       }
 
       case X_IS_INTERNAL:
       {
-         proclevel oldlevel ;
-         int stackmark ;
          nodeptr entry ;
          paramboxptr args ;
          streng *ptr ;
 
-         if ((entry=this->u.node)!=NULL)
+         /*
+          * Check if the internal function name ends with a '.'.
+          * This is an error in the ANSI standard, but it is possible
+          * that existing code allows this, so only generate an error
+          * if STRICT_ANSI OPTION is set.
+          */
+         if ( get_options_flag( TSD->currlevel, EXT_STRICT_ANSI ) )
          {
-            nodeptr savecurrentnode;  /* pgb */
-            set_sigl( TSD, TSD->currentnode->lineno ) ;
-            oldlevel = TSD->currlevel ;
-            args = initplist( TSD, this ) ;
-            TSD->currlevel = newlevel( TSD, TSD->currlevel ) ;
-            TSD->currlevel->args = args ;
-            stackmark = pushcallstack(TSD,TSD->currentnode) ;
+            if ( this->name->value[(this->name->len)-1] == '.' )
+               exiterror( ERR_UNQUOTED_FUNC_STOP, 1, tmpstr_of( TSD, this->name ) )  ;
+         }
+         if ( ( entry = this->u.node ) != NULL )
+         {
+            set_reserved_value( TSD, POOL0_SIGL, NULL,
+                                TSD->currentnode->lineno, VFLAG_NUM );
+            args = initplist( TSD, this );
 
-            savecurrentnode = TSD->currentnode; /* pgb */
-            ptr = interpret( TSD, entry->next ) ;
-            TSD->currentnode = savecurrentnode; /* pgb */
+            ptr = CallInternalFunction( TSD, entry->next, TSD->currentnode,
+                                        args );
 
-            if (ptr==NULL)
-                exiterror( ERR_NO_DATA_RETURNED, 0 )  ;
+            if (ptr==NULL) /* fixes bug 592393 */
+               exiterror( ERR_NO_DATA_RETURNED, 1, tmpstr_of( TSD, this->name ) );
 
-            popcallstack(TSD,stackmark) ;
-            removelevel( TSD, TSD->currlevel ) ;
-            TSD->currlevel = oldlevel ;
-            TSD->currlevel->next = NULL ;
-            TSD->trace_stat = TSD->currlevel->tracestat ;
-            if (kill)
-               *kill = ptr ;
             if (TSD->trace_stat=='I')
-               tracevalue( TSD, ptr, 'F' ) ;
+               tracevalue( TSD, ptr, 'F' );
 
-            return ptr ;
+            RETURN_NEW( ptr );
          }
       }
-
+      /* THIS IS MEANT TO FALL THROUGH! */
       case X_IS_BUILTIN:
       case X_EX_FUNC:
       {
@@ -479,113 +554,92 @@ cnsnt:   if (kill)
          {
             if (this->type != X_IS_BUILTIN)
                this->type = X_IS_BUILTIN ;
-            if (kill)
-               *kill = ptr ;
 
             if (!ptr)
-                exiterror( ERR_NO_DATA_RETURNED, 0 )  ;
+                exiterror( ERR_NO_DATA_RETURNED, 1, tmpstr_of( TSD, this->name ) )  ;
 
             if (TSD->trace_stat=='I')
                tracevalue( TSD, ptr, 'F' ) ;
 
-            return ptr ;
+            RETURN_NEW( ptr );
          }
          else
             this->type = X_IS_EXTERNAL ;
       }
-
+      /* THIS IS MEANT TO FALL THROUGH! */
       case X_IS_EXTERNAL:
       {
-         streng *ptr, *command ;
-         int stackmark ;
-         paramboxptr args, targs ;
+         streng *ptr, *command;
+         int stackmark,len,err;
+         paramboxptr args, targs;
 
-         update_envirs( TSD, TSD->currlevel ) ;
+         if ( TSD->restricted )
+            exiterror( ERR_RESTRICTED, 5 );
 
-         args = targs = initplist( TSD, this ) ;
-#if 0
-         command = Str_makeTSD( 1000 ) ;
-         command = Str_catTSD(command,this->name ) ;
-         for (;targs;targs=targs->next)
-            if (targs->value)
-            {
-               command = Str_catstrTSD(command," ") ;
-               command = Str_catTSD(command,targs->value) ;
-            }
+         update_envirs( TSD, TSD->currlevel );
 
-         stackmark = pushcallstack( TSD, TSD->currentnode ) ;
-         ptr = execute_external(TSD,command,args,TSD->systeminfo->environment,
-                                NULL,0, INVO_FUNCTION);
-         popcallstack( TSD, stackmark ) ;
-         deallocplink( TSD, args ) ;
+         args = initplist( TSD, this );
+         stackmark = pushcallstack( TSD, TSD->currentnode );
+         ptr = execute_external( TSD, this->name,
+                                 args,
+                                 TSD->systeminfo->environment,
+                                 &err,
+                                 TSD->systeminfo->hooks,
+                                 INVO_FUNCTION );
+         popcallstack( TSD, stackmark );
 
-         if (!ptr)
-            ptr = run_popen( TSD, command, TSD->currlevel->environment ) ;
-
-         if (!ptr)
-         {
-             exiterror( ERR_ROUTINE_NOT_FOUND, 0 )  ;
-            ptr = nullstringptr() ;
-         }
-         if (kill)
-            *kill = ptr ;
-
-         if (TSD->trace_stat=='I')
-            tracevalue( TSD, ptr, 'F' ) ;
-
-         Free_stringTSD( command ) ;
-#else
-
-         stackmark = pushcallstack( TSD, TSD->currentnode ) ;
-         ptr = execute_external(TSD,this->name,args,
-                                TSD->systeminfo->environment,NULL,0,
-                                INVO_FUNCTION);
-         popcallstack( TSD, stackmark ) ;
-
-         if (!ptr)
+         if ( err == -ERR_PROG_UNREADABLE )
          {
             /*
              * "this->name" wasn't a Rexx program, so
              * see if it is an OS command.
              * Only do this if the OPTIONS EXT_COMMANDS_AS_FUNCS is
-             * set.
+             * set and STRICT_ANSI is NOT set.
              */
-#if OLD_OPTIONS
-            if ( TSD->currlevel->u.options.ext_commands_as_funcs )
-#else
-            if ( get_options_flag( TSD->currlevel, EXT_EXT_COMMANDS_AS_FUNCS ) )
-#endif
+            if ( get_options_flag( TSD->currlevel, EXT_EXT_COMMANDS_AS_FUNCS )
+            &&  !get_options_flag( TSD->currlevel, EXT_STRICT_ANSI ) )
             {
-               command = Str_makeTSD( 1000 ) ;
-               command = Str_catTSD(command,this->name ) ;
-               for (;targs;targs=targs->next)
+               len = Str_len( this->name );
+               for( targs = args; targs; targs = targs->next )
                {
-                  if (targs->value)
+                  if ( targs->value )
+                     len += 1 + Str_len( targs->value );
+               }
+               command = Str_makeTSD( len );
+               command = Str_catTSD( command, this->name );
+               for( targs = args; targs; targs = targs->next )
+               {
+                  if ( targs->value )
                   {
-                     command = Str_catstrTSD(command," ") ;
-                     command = Str_catTSD(command,targs->value) ;
+                     command = Str_catstrTSD( command, " " );
+                     command = Str_catTSD( command, targs->value );
                   }
                }
-               ptr = run_popen( TSD, command, TSD->currlevel->environment ) ;
-               Free_stringTSD( command ) ;
+               ptr = run_popen( TSD, command, TSD->currlevel->environment );
+               if ( ptr != NULL )
+                  err = 0;
+               Free_stringTSD( command );
             }
          }
 
-         deallocplink( TSD, args ) ;
+         deallocplink( TSD, args );
 
-         if (!ptr)
+         if ( ptr && ( TSD->trace_stat == 'I' ) )
+            tracevalue( TSD, ptr, 'F' );
+
+         if ( err == -ERR_PROG_UNREADABLE )
          {
-            exiterror( ERR_ROUTINE_NOT_FOUND, 1, tmpstr_of( TSD, this->name ) )  ;
-            ptr = nullstringptr() ;
+            exiterror( ERR_ROUTINE_NOT_FOUND, 1, tmpstr_of( TSD, this->name ) );
+         }
+         else if ( err )
+         {
+            post_process_system_call( TSD, this->name, -err, NULL, this );
          }
-         if (kill)
-            *kill = ptr ;
 
-         if (TSD->trace_stat=='I')
-            tracevalue( TSD, ptr, 'F' ) ;
+         if ( !ptr )
+            exiterror( ERR_NO_DATA_RETURNED, 1, tmpstr_of( TSD, this->name ) );
 
-#endif
-         return ptr ;
+         RETURN_NEW( ptr );
       }
 
       case X_CONCAT:
@@ -605,18 +659,14 @@ cnsnt:   if (kill)
          memcpy( cptr, strtwo->value, strtwo->len ) ;
          strthr->len = (cptr-strthr->value) + strtwo->len ;
 
-         if (kill)
-            *kill = strthr ;
-         if (stmp1)
-            Free_stringTSD( stmp1 ) ;
-         if (stmp2)
-            Free_stringTSD( stmp2 ) ;
+         FREE_TMP_STRING( stmp1 );
+         FREE_TMP_STRING( stmp2 );
 
          if (TSD->trace_stat=='I')
             tracevalue( TSD, strthr, 'O' ) ;
 
 
-         return strthr ;
+         RETURN_NEW( strthr );
       }
 
 
@@ -650,28 +700,30 @@ cnsnt:   if (kill)
       case X_S_GTE:
       case X_S_LT:
       case X_S_LTE:
-         sptr = bool_to_str( TSD, isboolean( TSD, this )) ;
-         if (kill)
-            *kill = sptr ;
-         return sptr ;
+         stmp1 = bool_to_str( TSD, isboolean( TSD, this )) ;
+         RETURN_NEW( stmp1 );
 
       default:
-          exiterror( ERR_INTERPRETER_FAILURE, 0 )  ;
+         exiterror( ERR_INTERPRETER_FAILURE, 1, __FILE__, __LINE__, "" )  ;
          return NULL ;
    }
+#undef RETURN_NEW
 }
 
 
-
+/*
+ * isboolean evaluates a boolean expression and returns 0 for false, another
+ * value for true. "this" is the current evaluation tree.
+ * Note: This is one of the most time-consuming routines. Be careful.
+ */
 int isboolean( tsd_t *TSD, nodeptr this )
 {
-   streng *strone=NULL, *strtwo=NULL ;
-   streng *stmp1=NULL, *stmp2=NULL ;
-   int tmp=0, sint=0 ;
-   streng *sptr=NULL ;
-   num_descr *ntmp=NULL ;
+   streng *strone,*strtwo;
+   streng *stmp1,*stmp2;
+   int tmp,sint;
+   num_descr *ntmp;
 
-   switch (/*(unsigned char)*/(this->type))
+   switch ( this->type )
    {
       case 0:
       case 255:
@@ -684,6 +736,7 @@ int isboolean( tsd_t *TSD, nodeptr this )
       case X_EXP:
       case X_U_MINUS:
       case X_U_PLUSS:
+         ntmp = NULL;
          tmp = num_to_bool( calcul( TSD, this, &ntmp )) ;
          if (ntmp)
          {
@@ -694,7 +747,7 @@ int isboolean( tsd_t *TSD, nodeptr this )
 
       case X_STRING:
       case X_CON_SYMBOL:
-         if (!this->u.number)
+         if ( !this->u.number )
             this->u.number = get_a_descr( TSD, this->name ) ;
          return num_to_bool( this->u.number ) ;
 
@@ -703,7 +756,7 @@ int isboolean( tsd_t *TSD, nodeptr this )
          return num_to_bool( shortcutnum( TSD, this )) ;
 
       case X_HEAD_SYMBOL:
-         return num_to_bool( fix_compoundnum( TSD, this, NULL )) ;
+         return num_to_bool( fix_compoundnum( TSD, this, NULL, NULL ) );
 
       case X_IN_FUNC:
       case X_IS_INTERNAL:
@@ -712,10 +765,9 @@ int isboolean( tsd_t *TSD, nodeptr this )
       case X_IS_EXTERNAL:
       case X_CONCAT:
       case X_SPACE:
-         tmp = str_to_bool( sptr = evaluate( TSD, this, &sptr )) ;
-         if (sptr)
-            Free_stringTSD( sptr ) ;
-         return tmp ;
+         tmp = str_to_bool( evaluate( TSD, this, &stmp1 ) );
+         FREE_TMP_STRING( stmp1 );
+         return tmp;
 
       case X_LOG_NOT:
          sint = !isboolean( TSD, this->p[0] ) ;
@@ -761,6 +813,9 @@ int isboolean( tsd_t *TSD, nodeptr this )
 
          if (flags.lnum)
          {
+            if ( !this->p[0]->u.number )
+               this->p[0]->u.number = get_a_descr( TSD, this->p[0]->name );
+
             lnum = this->p[0]->u.number ;
             if (TSD->trace_stat=='I')
                tracenumber( TSD, lnum, 'L' ) ;
@@ -768,13 +823,16 @@ int isboolean( tsd_t *TSD, nodeptr this )
          else if (flags.lsvar)
             lnum = shortcutnum( TSD, this->p[0] ) ;
          else if (flags.lcvar)
-            lnum = fix_compoundnum( TSD, this->p[0], NULL ) ;
+            lnum = fix_compoundnum( TSD, this->p[0], NULL, NULL );
 
          if (!lnum)
             lval = evaluate( TSD, this->p[0], &stmp1 ) ;
 
          if (flags.rnum)
          {
+            if ( !this->p[1]->u.number )
+               this->p[1]->u.number = get_a_descr( TSD, this->p[1]->name );
+
             rnum = this->p[1]->u.number ;
             if (TSD->trace_stat=='I')
                tracenumber( TSD, rnum, 'L' ) ;
@@ -782,7 +840,7 @@ int isboolean( tsd_t *TSD, nodeptr this )
          else if (flags.rsvar)
             rnum = shortcutnum( TSD, this->p[1] ) ;
          else if (flags.rcvar)
-            rnum = fix_compoundnum( TSD, this->p[1], NULL ) ;
+            rnum = fix_compoundnum( TSD, this->p[1], NULL, NULL );
 
          if (!rnum)
             rval = evaluate( TSD, this->p[1], &stmp2 ) ;
@@ -797,52 +855,37 @@ int isboolean( tsd_t *TSD, nodeptr this )
             tmp = string_test( TSD, lnum, rnum ) ;
          else
          {
-            char *s1, *s2, *e1, *e2 ;
+            unsigned char *s1,*s2,*e1,*e2;
 
-            if (!lval)
+            if ( !lval )
             {
-               assert( !stmp1 ) ;
-               stmp1 = lval = str_norm( TSD, lnum, NULL ) ;
+               assert( !stmp1 );
+               stmp1 = lval = str_norm( TSD, lnum, NULL );
             }
 
-            if (!rval)
+            if ( !rval )
             {
-               assert( !stmp2 ) ;
-               stmp2 = rval = str_norm( TSD, rnum, NULL ) ;
+               assert( !stmp2 );
+               stmp2 = rval = str_norm( TSD, rnum, NULL );
             }
 
-            s1 = lval->value ;
-            s2 = rval->value ;
-            e1 = s1 + lval->len ;
-            e2 = s2 + rval->len ;
-            /*
-             * Strip leading spaces - ignored in comparison
-             */
-            for(;(s1<e1)&&(isspace(*s1));s1++);
-            for(;(s2<e2)&&(isspace(*s2));s2++);
-            for (;(s1<e1)&&(s2<e2)&&(*s1==*s2);s1++,s2++) ;
-            /*
-             * Strip trailing spaces - ignored in comparison
-             */
-            /*  wait a bit! it is not (isspace(*s1)) we should test , but the end of string!  bja
-            for (;(e1>s1)&&(isspace(*s1));e1--) ;
-            for (;(e2>s2)&&(isspace(*s2));e2--) ;
-            */
-            for (;(e1>s1)&&(isspace(*(e1-1)));e1--) ;                                      /* bja */
-            for (;(e2>s2)&&(isspace(*(e2-1)));e2--) ;                                      /* bja */
+            s1 = (unsigned char *) lval->value;
+            s2 = (unsigned char *) rval->value;
+            e1 = (unsigned char *) s1 + lval->len;
+            e2 = (unsigned char *) s2 + rval->len;
 
-            if (s1==e1 && s2==e2)
-               tmp = 0 ;
-            else if (s1<e1 && s2<e2)
-               tmp = (*s1<*s2) ? -1 : 1 ;
+            strip_whitespace( TSD, &s1, &e1, &s2, &e2 );
+
+            if ( s1 == e1 && s2 == e2 )
+               tmp = 0;
+            else if ( s1 < e1 && s2 < e2 )
+               tmp = ( *s1 < *s2 ) ? -1 : 1;
             else
-               tmp = (s1<e1) ? 1 : -1 ;
+               tmp = ( s1 < e1 ) ? 1 : -1;
          }
 
-         if (stmp1)
-            Free_stringTSD( stmp1 ) ;
-         if (stmp2)
-            Free_stringTSD( stmp2 ) ;
+         FREE_TMP_STRING( stmp1 );
+         FREE_TMP_STRING( stmp2 );
 
          type = this->type ;
          if (tmp==0)
@@ -865,32 +908,19 @@ int isboolean( tsd_t *TSD, nodeptr this )
       case X_SEQUAL:
       case X_SDIFF:
       {  /* string comparison */
-         char *s1, *s2, *e1, *e2 ;
+         unsigned char *s1, *s2, *e1, *e2 ;
          int type ;
 
          type = this->type ;
          strone = evaluate( TSD, this->p[0], &stmp1 ) ;
          strtwo = evaluate( TSD, this->p[1], &stmp2 ) ;
 
-         s1 = strone->value ;
-         s2 = strtwo->value ;
-         e1 = s1 + strone->len ;
-         e2 = s2 + strtwo->len ;
-         /*
-          * Strip leading spaces - ignored in comparison
-          */
-         for(;(s1<e1)&&(isspace(*s1));s1++);
-         for(;(s2<e2)&&(isspace(*s2));s2++);
-         for (;(s1<e1)&&(s2<e2)&&(*s1==*s2);s1++,s2++) ;
-         /*
-          * Strip trailing spaces - ignored in comparison
-          */
-         /*  wait a bit! it is not (isspace(*s1)) we should test , but the end of string!     bja
-         for (;(e1>s1)&&(isspace(*s1));e1--) ;
-         for (;(e2>s2)&&(isspace(*s2));e2--) ;
-         */                                                                                /* bja */
-         for (;(e1>s1)&&(isspace(*(e1-1)));e1--) ;                                         /* bja */
-         for (;(e2>s2)&&(isspace(*(e2-1)));e2--) ;                                         /* bja */
+         s1 = (unsigned char *) strone->value ;
+         s2 = (unsigned char *) strtwo->value ;
+         e1 = (unsigned char *) s1 + strone->len ;
+         e2 = (unsigned char *) s2 + strtwo->len ;
+
+         strip_whitespace( TSD, &s1, &e1, &s2, &e2 );
 
          if (s1==e1 && s2==e2)
             tmp = 0 ;
@@ -899,10 +929,8 @@ int isboolean( tsd_t *TSD, nodeptr this )
          else
             tmp = (s1<e1) ? 1 : -1 ;
 
-         if (stmp1)
-            Free_stringTSD( stmp1 ) ;
-         if (stmp2)
-            Free_stringTSD( stmp2 ) ;
+         FREE_TMP_STRING( stmp1 );
+         FREE_TMP_STRING( stmp2 );
 
          if (tmp==0)
             sint = (type==X_SGTE || type==X_SLTE || type==X_SEQUAL) ;
@@ -931,6 +959,7 @@ int isboolean( tsd_t *TSD, nodeptr this )
 
          type = this->type ;
 
+         ntmp1 = ntmp2 = NULL;
          numone = calcul( TSD, this->p[0], &ntmp1 ) ;
          numtwo = calcul( TSD, this->p[1], &ntmp2 ) ;
          tmp = string_test( TSD, numone, numtwo ) ;
@@ -963,10 +992,10 @@ int isboolean( tsd_t *TSD, nodeptr this )
          strone = evaluate( TSD, this->p[0], &stmp1 ) ;
          strtwo = evaluate( TSD, this->p[1], &stmp2 ) ;
          tmp = Str_cmp(strone,strtwo)!=0 ;
-         if (stmp1)
-            Free_stringTSD( stmp1 ) ;
-         if (stmp2)
-            Free_stringTSD( stmp2 ) ;
+
+         FREE_TMP_STRING( stmp1 );
+         FREE_TMP_STRING( stmp2 );
+
          if (TSD->trace_stat=='I')
             tracebool( TSD, tmp, 'O' ) ;
 
@@ -977,10 +1006,10 @@ int isboolean( tsd_t *TSD, nodeptr this )
          strone = evaluate( TSD, this->p[0], &stmp1 ) ;
          strtwo = evaluate( TSD, this->p[1], &stmp2 ) ;
          tmp = Str_cmp(strone,strtwo)==0 ;
-         if (stmp1)
-            Free_stringTSD( stmp1 ) ;
-         if (stmp2)
-            Free_stringTSD( stmp2 ) ;
+
+         FREE_TMP_STRING( stmp1 );
+         FREE_TMP_STRING( stmp2 );
+
          if (TSD->trace_stat=='I')
             tracebool( TSD, tmp, 'O' ) ;
 
@@ -989,17 +1018,17 @@ int isboolean( tsd_t *TSD, nodeptr this )
       case X_S_NGT:
       case X_S_NLT:
       {  /* strict string NOT comparison */
-         char *s1, *s2, *e1, *e2 ;
+         unsigned char *s1, *s2, *e1, *e2 ;
          int type ;
 
          type = this->type ;
          strone = evaluate( TSD, this->p[0], &stmp1 ) ;
          strtwo = evaluate( TSD, this->p[1], &stmp2 ) ;
 
-         s1 = strone->value ;
-         s2 = strtwo->value ;
-         e1 = s1 + strone->len ;
-         e2 = s2 + strtwo->len ;
+         s1 = (unsigned char *) strone->value ;
+         s2 = (unsigned char *) strtwo->value ;
+         e1 = (unsigned char *) s1 + strone->len ;
+         e2 = (unsigned char *) s2 + strtwo->len ;
          /*
           * same compare as non-strict except that leading and trailing spaces
           * are retained for comparison.
@@ -1012,10 +1041,8 @@ int isboolean( tsd_t *TSD, nodeptr this )
          else
             tmp = (s1<e1) ? -1 : 1 ;
 
-         if (stmp1)
-            Free_stringTSD( stmp1 ) ;
-         if (stmp2)
-            Free_stringTSD( stmp2 ) ;
+         FREE_TMP_STRING( stmp1 );
+         FREE_TMP_STRING( stmp2 );
 
          if (tmp==0)
             sint = 1;
@@ -1035,17 +1062,17 @@ int isboolean( tsd_t *TSD, nodeptr this )
       case X_S_LT:
       case X_S_LTE:
       {  /* strict string comparison */
-         char *s1, *s2, *e1, *e2 ;
+         unsigned char *s1, *s2, *e1, *e2 ;
          int type ;
 
          type = this->type ;
          strone = evaluate( TSD, this->p[0], &stmp1 ) ;
          strtwo = evaluate( TSD, this->p[1], &stmp2 ) ;
 
-         s1 = strone->value ;
-         s2 = strtwo->value ;
-         e1 = s1 + strone->len ;
-         e2 = s2 + strtwo->len ;
+         s1 = (unsigned char *) strone->value ;
+         s2 = (unsigned char *) strtwo->value ;
+         e1 = (unsigned char *) s1 + strone->len ;
+         e2 = (unsigned char *) s2 + strtwo->len ;
          /*
           * same compare as non-strict except that leading and trailing spaces
           * are retained for comparison.
@@ -1058,10 +1085,8 @@ int isboolean( tsd_t *TSD, nodeptr this )
          else
             tmp = (s1<e1) ? 1 : -1 ;
 
-         if (stmp1)
-            Free_stringTSD( stmp1 ) ;
-         if (stmp2)
-            Free_stringTSD( stmp2 ) ;
+         FREE_TMP_STRING( stmp1 );
+         FREE_TMP_STRING( stmp2 );
 
          if (tmp==0)
             sint = (type==X_S_GTE || type==X_S_LTE) ;
@@ -1078,7 +1103,7 @@ int isboolean( tsd_t *TSD, nodeptr this )
 
 
       default:
-          exiterror( ERR_INTERPRETER_FAILURE, 0 )  ;
+         exiterror( ERR_INTERPRETER_FAILURE, 1, __FILE__, __LINE__, "" )  ;
          return 0 ;
    }
 }
