@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid = "$Id: funcs.c,v 1.6 2000/11/19 06:27:50 mark Exp $";
+static char *RCSid = "$Id: funcs.c,v 1.37 2004/04/15 10:08:45 mark Exp $";
 #endif
 
 /*
@@ -24,184 +24,216 @@ static char *RCSid = "$Id: funcs.c,v 1.6 2000/11/19 06:27:50 mark Exp $";
 #include "rexx.h"
 #include "rxiface.h"
 #include <string.h>
-#include <ctype.h>
 #include <stdio.h>
 #include <assert.h>
 #include <time.h>
 
+static streng *conflict_close( tsd_t *TSD, cparamboxptr parms );
+static streng *conflict_eof( tsd_t *TSD, cparamboxptr parms );
+static streng *conflict_open( tsd_t *TSD, cparamboxptr parms );
+
 struct function_type
 {
-   int length ;
+   int compat ;
    streng *(*function)(tsd_t *,cparamboxptr) ;
    const char *funcname ;
 } ;
 
+/*
+ * A 0 in the first column of this table indicates that this BIF is ANSI.
+ * Any other value
+ */
 static const struct function_type functions[] = {
-  { 0,              std_abbrev, "ABBREV" },
-  { 0,              std_abs, "ABS" },
-  { 0,              std_address, "ADDRESS" },
+  { 0,              std_abbrev,            "ABBREV" },
+  { 0,              std_abs,               "ABS" },
+  { 0,              std_address,           "ADDRESS" },
 #ifdef TRACEMEM
-  { 0,              dbg_allocated, "ALLOCATED" },
+  { EXT_REGINA_BIFS,dbg_allocated,         "ALLOCATED" },
 #endif
-  { 0,              std_arg, "ARG" },
-  { 0,              std_b2x, "B2X" },
-  { 0,              os2_beep, "BEEP" },
-  { 0,              std_bitand, "BITAND" },
-  { 0,              std_bitor, "BITOR" },
-  { 0,              std_bitxor, "BITXOR" },
-  { EXT_BUFTYPE_BIF,cms_buftype, "BUFTYPE" },
-  { 0,              std_c2d, "C2D" },
-  { 0,              std_c2x, "C2X" },
-  { 0,              unx_chdir, "CD" },
-  { 0,              std_center, "CENTER" },
-  { 0,              std_center, "CENTRE" },
-  { 0,              std_changestr, "CHANGESTR" }, /* ANSI Std 1996 - MH 10-06-96 */
-  { 0,              std_charin, "CHARIN" },
-  { 0,              std_charout, "CHAROUT" },
-  { 0,              std_chars, "CHARS" },
-  { 0,              unx_chdir, "CHDIR" },
-#ifdef OLD_REGINA_FEATURES
-  { 0,              unx_close, "CLOSE" },
-#endif
-  { 0,              std_compare, "COMPARE" },
-  { 0,              std_condition, "CONDITION" },
-  { 0,              std_copies, "COPIES" },
-  { 0,              std_countstr, "COUNTSTR" },   /* ANSI Std 1996 - MH 10-06-96 */
-  { 0,              unx_crypt, "CRYPT" },
-  { 0,              std_d2c, "D2C" },
-  { 0,              std_d2x, "D2X" },
-  { 0,              std_datatype, "DATATYPE" },
-  { 0,              std_date, "DATE" },
-  { 0,              std_delstr, "DELSTR" },
-  { 0,              std_delword, "DELWORD" },
-  { EXT_DESBUF_BIF, cms_desbuf, "DESBUF" },
-  { 0,              std_digits, "DIGITS" },
-  { 0,              os2_directory, "DIRECTORY" },
-  { EXT_DROPBUF_BIF,cms_dropbuf, "DROPBUF" },
+  { 0,              std_arg,               "ARG" },
+  { EXT_AREXX_BIFS, arexx_b2c,             "B2C" },
+  { 0,              std_b2x,               "B2X" },
+  { EXT_REGINA_BIFS,os2_beep,              "BEEP" },
+  { 0,              std_bitand,            "BITAND" },
+  { EXT_AREXX_BIFS, arexx_bitchg,          "BITCHG" },
+  { EXT_AREXX_BIFS, arexx_bitclr,          "BITCLR" },
+  { EXT_AREXX_BIFS, arexx_bitcomp,         "BITCOMP" },
+  { 0,              std_bitor,             "BITOR" },
+  { EXT_AREXX_BIFS, arexx_bitset,          "BITSET" },
+  { EXT_AREXX_BIFS, arexx_bittst,          "BITTST" },
+  { 0,              std_bitxor,            "BITXOR" },
+  { EXT_BUFTYPE_BIF,cms_buftype,           "BUFTYPE" },
+  { EXT_AREXX_BIFS, arexx_c2b,             "C2B" },
+  { 0,              std_c2d,               "C2D" },
+  { 0,              std_c2x,               "C2X" },
+  { EXT_REGINA_BIFS,unx_chdir,             "CD" },
+  { 0,              std_center,            "CENTER" },
+  { 0,              std_center,            "CENTRE" },
+  { 0,              std_changestr,         "CHANGESTR" }, /* ANSI Std 1996 - MH 10-06-96 */
+  { 0,              std_charin,            "CHARIN" },
+  { 0,              std_charout,           "CHAROUT" },
+  { 0,              std_chars,             "CHARS" },
+  { EXT_REGINA_BIFS,unx_chdir,             "CHDIR" },
+  { EXT_REGINA_BIFS,conflict_close,        "CLOSE" },
+  { 0,              std_compare,           "COMPARE" },
+  { EXT_AREXX_BIFS, arexx_compress,        "COMPRESS" },
+  { 0,              std_condition,         "CONDITION" },
+  { 0,              std_copies,            "COPIES" },
+  { 0,              std_countstr,          "COUNTSTR" },   /* ANSI Std 1996 - MH 10-06-96 */
+  { EXT_REGINA_BIFS,unx_crypt,             "CRYPT" },
+  { 0,              std_d2c,               "D2C" },
+  { 0,              std_d2x,               "D2X" },
+  { 0,              std_datatype,          "DATATYPE" },
+  { 0,              std_date,              "DATE" },
+  { 0,              std_delstr,            "DELSTR" },
+  { 0,              std_delword,           "DELWORD" },
+  { EXT_DESBUF_BIF, cms_desbuf,            "DESBUF" },
+  { 0,              std_digits,            "DIGITS" },
+  { EXT_REGINA_BIFS,os2_directory,         "DIRECTORY" },
+  { EXT_DROPBUF_BIF,cms_dropbuf,           "DROPBUF" },
 #ifndef NDEBUG
-  { 0,              dbg_dumpfiles, "DUMPFILES" },
-  { 0,              dbg_dumptree, "DUMPTREE" },
-  { 0,              dbg_dumpvars, "DUMPVARS" },
+  { EXT_REGINA_BIFS,dbg_dumpfiles,         "DUMPFILES" },
+  { EXT_REGINA_BIFS,dbg_dumptree,          "DUMPTREE" },
+  { EXT_REGINA_BIFS,dbg_dumpvars,          "DUMPVARS" },
 #endif
-  { 0,              unx_eof, "EOF" },
-  { 0,              std_errortext, "ERRORTEXT" },
+  { EXT_REGINA_BIFS,conflict_eof,          "EOF" },
+  { 0,              std_errortext,         "ERRORTEXT" },
+  { EXT_AREXX_BIFS, arexx_exists,          "EXISTS" },
+  { EXT_AREXX_BIFS, arexx_export,          "EXPORT" },
 #ifdef VMS
-  { 0,              vms_f_cvsi, "F$CVSI" },
-  { 0,              vms_f_cvtime, "F$CVTIME" },
-  { 0,              vms_f_cvui, "F$CVUI" },
-  { 0,              vms_f_directory, "F$DIRECTORY" },
-  { 0,              vms_f_element, "F$ELEMENT" },
-  { 0,              vms_f_extract, "F$EXTRACT" },
-  { 0,              vms_f_fao, "F$FAO" },
-  { 0,              vms_f_file_attributes, "F$FILE_ATTRIBUTES" },
-  { 0,              vms_f_getdvi, "F$GETDVI" },
-  { 0,              vms_f_getjpi, "F$GETJPI" },
-  { 0,              vms_f_getqui, "F$GETQUI" },
-  { 0,              vms_f_getsyi, "F$GETSYI" },
-  { 0,              vms_f_identifier, "F$IDENTIFIER" },
-  { 0,              vms_f_integer, "F$INTEGER" },
-  { 0,              vms_f_length, "F$LENGTH" },
-  { 0,              vms_f_locate, "F$LOCATE" },
-  { 0,              vms_f_logical, "F$LOGICAL" },
-  { 0,              vms_f_message, "F$MESSAGE" },
-  { 0,              vms_f_mode, "F$MODE" },
-  { 0,              vms_f_parse, "F$PARSE" },
-  { 0,              vms_f_pid, "F$PID" },
-  { 0,              vms_f_privilege, "F$PRIVILEGE" },
-  { 0,              vms_f_process, "F$PROCESS" },
-  { 0,              vms_f_search, "F$SEARCH" },
-  { 0,              vms_f_setprv, "F$SETPRV" },
-  { 0,              vms_f_string, "F$STRING" },
-  { 0,              vms_f_time, "F$TIME" },
-  { 0,              vms_f_trnlnm, "F$TRNLNM" },
-  { 0,              vms_f_type, "F$TYPE" },
-  { 0,              vms_f_user, "F$USER" },
-/*{ 0,              vms_f_verify, "F$VERIFY" }, */
+  { EXT_REGINA_BIFS,vms_f_cvsi,            "F$CVSI" },
+  { EXT_REGINA_BIFS,vms_f_cvtime,          "F$CVTIME" },
+  { EXT_REGINA_BIFS,vms_f_cvui,            "F$CVUI" },
+  { EXT_REGINA_BIFS,vms_f_directory,       "F$DIRECTORY" },
+  { EXT_REGINA_BIFS,vms_f_element,         "F$ELEMENT" },
+  { EXT_REGINA_BIFS,vms_f_extract,         "F$EXTRACT" },
+  { EXT_REGINA_BIFS,vms_f_fao,             "F$FAO" },
+  { EXT_REGINA_BIFS,vms_f_file_attributes, "F$FILE_ATTRIBUTES" },
+  { EXT_REGINA_BIFS,vms_f_getdvi,          "F$GETDVI" },
+  { EXT_REGINA_BIFS,vms_f_getjpi,          "F$GETJPI" },
+  { EXT_REGINA_BIFS,vms_f_getqui,          "F$GETQUI" },
+  { EXT_REGINA_BIFS,vms_f_getsyi,          "F$GETSYI" },
+  { EXT_REGINA_BIFS,vms_f_identifier,      "F$IDENTIFIER" },
+  { EXT_REGINA_BIFS,vms_f_integer,         "F$INTEGER" },
+  { EXT_REGINA_BIFS,vms_f_length,          "F$LENGTH" },
+  { EXT_REGINA_BIFS,vms_f_locate,          "F$LOCATE" },
+  { EXT_REGINA_BIFS,vms_f_logical,         "F$LOGICAL" },
+  { EXT_REGINA_BIFS,vms_f_message,         "F$MESSAGE" },
+  { EXT_REGINA_BIFS,vms_f_mode,            "F$MODE" },
+  { EXT_REGINA_BIFS,vms_f_parse,           "F$PARSE" },
+  { EXT_REGINA_BIFS,vms_f_pid,             "F$PID" },
+  { EXT_REGINA_BIFS,vms_f_privilege,       "F$PRIVILEGE" },
+  { EXT_REGINA_BIFS,vms_f_process,         "F$PROCESS" },
+  { EXT_REGINA_BIFS,vms_f_search,          "F$SEARCH" },
+  { EXT_REGINA_BIFS,vms_f_setprv,          "F$SETPRV" },
+  { EXT_REGINA_BIFS,vms_f_string,          "F$STRING" },
+  { EXT_REGINA_BIFS,vms_f_time,            "F$TIME" },
+  { EXT_REGINA_BIFS,vms_f_trnlnm,          "F$TRNLNM" },
+  { EXT_REGINA_BIFS,vms_f_type,            "F$TYPE" },
+  { EXT_REGINA_BIFS,vms_f_user,            "F$USER" },
+/*{ EXT_REGINA_BIFS,vms_f_verify,          "F$VERIFY" }, */
 #endif
-  { 0,              os2_filespec, "FILESPEC" },
-  { EXT_FIND_BIF,   cms_find, "FIND" },
+  { EXT_REGINA_BIFS,os2_filespec,          "FILESPEC" },
+  { EXT_REGINA_BIFS,cms_find,              "FIND" },
 #ifdef OLD_REGINA_FEATURES
-  { 0,              unx_close, "FINIS" },
+  { EXT_REGINA_BIFS,unx_close,             "FINIS" },
 #endif /* OLD_REGINA_FEATURES */
-  { 0,              unx_fork, "FORK" },
-  { 0,              std_form, "FORM" },
-  { 0,              std_format, "FORMAT" },
+  { EXT_REGINA_BIFS,unx_fork,              "FORK" },
+  { 0,              std_form,              "FORM" },
+  { 0,              std_format,            "FORMAT" },
 #if defined(REGINA_DEBUG_MEMORY)
-  { 0,              dbg_freelists, "FREELISTS" },
+  { EXT_REGINA_BIFS,dbg_freelists,         "FREELISTS" },
 #endif
-  { 0,              std_fuzz, "FUZZ" },
-  { 0,              unx_getenv, "GETENV" },
-  { 0,              unx_getpath, "GETPATH" },
-  { 0,              unx_getpid, "GETPID" },
-  { 0,              unx_gettid, "GETTID" },
-  { 0,              cms_index, "INDEX" },
-  { 0,              std_insert, "INSERT" },
-  { 0,              cms_justify, "JUSTIFY" },
-  { 0,              std_lastpos, "LASTPOS" },
-  { 0,              std_left, "LEFT" },
-  { 0,              std_length, "LENGTH" },
-  { 0,              std_linein, "LINEIN" },
-  { 0,              std_lineout, "LINEOUT" },
-  { 0,              std_lines, "LINES" },
+  { EXT_AREXX_BIFS, arexx_freespace,       "FREESPACE" },
+  { 0,              std_fuzz,              "FUZZ" },
+  { EXT_REGINA_BIFS,unx_getenv,            "GETENV" },
+  { EXT_REGINA_BIFS,unx_getpath,           "GETPATH" },
+  { EXT_REGINA_BIFS,unx_getpid,            "GETPID" },
+  { EXT_AREXX_BIFS, arexx_getspace,        "GETSPACE" },
+  { EXT_REGINA_BIFS,unx_gettid,            "GETTID" },
+  { EXT_AREXX_BIFS, arexx_hash,            "HASH" },
+  { EXT_AREXX_BIFS, arexx_import,          "IMPORT" },
+  { EXT_REGINA_BIFS,cms_index,             "INDEX" },
+  { 0,              std_insert,            "INSERT" },
+  { EXT_REGINA_BIFS,cms_justify,           "JUSTIFY" },
+  { 0,              std_lastpos,           "LASTPOS" },
+  { 0,              std_left,              "LEFT" },
+  { 0,              std_length,            "LENGTH" },
+  { 0,              std_linein,            "LINEIN" },
+  { 0,              std_lineout,           "LINEOUT" },
+  { 0,              std_lines,             "LINES" },
 #ifdef TRACEMEM
-  { 0,              dbg_listleaked, "LISTLEAKED" },
+  { EXT_REGINA_BIFS,dbg_listleaked,        "LISTLEAKED" },
 #endif
-  { EXT_MAKEBUF_BIF,cms_makebuf, "MAKEBUF" },
-  { 0,              std_max, "MAX" },
+  { EXT_REGINA_BIFS,rex_lower,             "LOWER" },
+  { EXT_MAKEBUF_BIF,cms_makebuf,           "MAKEBUF" },
+  { 0,              std_max,               "MAX" },
 #ifdef TRACEMEM
-  { 0,              dbg_memorystats, "MEMORYSTATS" },
+  { EXT_REGINA_BIFS,dbg_memorystats,       "MEMORYSTATS" },
 #endif
-  { 0,              std_min, "MIN" },
-#ifdef OLD_REGINA_FEATURES
-  { EXT_OPEN_BIF,   unx_open, "OPEN" },
-#endif /* OLD_REGINA_FEATURES */
-  { 0,              std_overlay, "OVERLAY" },
-  { 0,              unx_popen, "POPEN" },
-  { 0,              std_pos, "POS" },
-  { 0,              std_qualify, "QUALIFY" },
-  { 0,              std_queued, "QUEUED" },
-  { 0,              std_random, "RANDOM" },
-  { 0,              std_reverse, "REVERSE" },
-  { 0,              std_right, "RIGHT" },
+  { 0,              std_min,               "MIN" },
+  { EXT_REGINA_BIFS,conflict_open,         "OPEN" },
+  { 0,              std_overlay,           "OVERLAY" },
+  { EXT_REGINA_BIFS,rex_poolid,            "POOLID" },
+  { EXT_REGINA_BIFS,unx_popen,             "POPEN" },
+  { 0,              std_pos,               "POS" },
+  { 0,              std_qualify,           "QUALIFY" },
+  { 0,              std_queued,            "QUEUED" },
+  { 0,              std_random,            "RANDOM" },
+  { EXT_AREXX_BIFS, arexx_randu,           "RANDU" },
+  { EXT_AREXX_BIFS, arexx_readch,          "READCH" },
+  { EXT_AREXX_BIFS, arexx_readln,          "READLN" },
+  { 0,              std_reverse,           "REVERSE" },
+  { 0,              std_right,             "RIGHT" },
 
-  { 0,              rex_rxfuncadd, "RXFUNCADD" },
-  { 0,              rex_rxfuncdrop, "RXFUNCDROP" },
-  { 0,              rex_rxfuncerrmsg, "RXFUNCERRMSG" },
-  { 0,              rex_rxfuncquery, "RXFUNCQUERY" },
-  { 0,              rex_rxqueue, "RXQUEUE" },
-
-  { 0,              std_sign, "SIGN" },
-  { 0,              cms_sleep, "SLEEP" },
-  { 0,              std_sourceline, "SOURCELINE" },
-  { 0,              std_space, "SPACE" },
-  { 0,              cms_state, "STATE" },
-  { 0,              std_stream, "STREAM" },
-  { 0,              std_strip, "STRIP" },
-  { 0,              std_substr, "SUBSTR" },
-  { 0,              std_subword, "SUBWORD" },
-  { 0,              std_symbol, "SYMBOL" },
-  { 0,              std_time, "TIME" },
-  { 0,              std_trace, "TRACE" },
-#ifndef NDEBUG
-  { 0,              dbg_traceback, "TRACEBACK" },
+  { 0,              rex_rxfuncadd,         "RXFUNCADD" },
+#ifdef HAVE_GCI
+  { EXT_REGINA_BIFS,rex_rxfuncdefine,      "RXFUNCDEFINE" },
 #endif
-  { 0,              std_translate, "TRANSLATE" },
-  { 0,              std_trunc, "TRUNC" },
-  { 0,              unx_uname, "UNAME" },
-  { 0,              unx_unixerror, "UNIXERROR" },
-  { 0,              rex_userid, "USERID" },
-  { 0,              std_value, "VALUE" },
-  { 0,              std_verify, "VERIFY" },
-  { 0,              std_word, "WORD" },
-  { 0,              std_wordindex, "WORDINDEX" },
-  { 0,              std_wordlength, "WORDLENGTH" },
-  { 0,              std_wordpos, "WORDPOS" },
-  { 0,              std_words, "WORDS" },
-  { 0,              std_x2b, "X2B" },
-  { 0,              std_x2c, "X2C" },
-  { 0,              std_x2d, "X2D" },
-  { 0,              std_xrange, "XRANGE" },
-  { 0,              NULL, NULL }
+  { 0,              rex_rxfuncdrop,        "RXFUNCDROP" },
+  { EXT_REGINA_BIFS,rex_rxfuncerrmsg,      "RXFUNCERRMSG" },
+  { 0,              rex_rxfuncquery,       "RXFUNCQUERY" },
+  { 0,              rex_rxqueue,           "RXQUEUE" },
+
+  { EXT_AREXX_BIFS, arexx_seek,            "SEEK" },
+  { EXT_AREXX_BIFS, arexx_show,            "SHOW" },
+  { 0,              std_sign,              "SIGN" },
+  { EXT_REGINA_BIFS,cms_sleep,             "SLEEP" },
+  { 0,              std_sourceline,        "SOURCELINE" },
+  { 0,              std_space,             "SPACE" },
+  { EXT_REGINA_BIFS,cms_state,             "STATE" },
+  { EXT_AREXX_BIFS, arexx_storage,         "STORAGE" },
+  { 0,              std_stream,            "STREAM" },
+  { 0,              std_strip,             "STRIP" },
+  { 0,              std_substr,            "SUBSTR" },
+  { 0,              std_subword,           "SUBWORD" },
+  { 0,              std_symbol,            "SYMBOL" },
+  { 0,              std_time,              "TIME" },
+  { 0,              std_trace,             "TRACE" },
+
+  { EXT_REGINA_BIFS,dbg_traceback,         "TRACEBACK" },
+
+  { 0,              std_translate,         "TRANSLATE" },
+  { EXT_AREXX_BIFS, arexx_trim,            "TRIM" },
+  { 0,              std_trunc,             "TRUNC" },
+  { EXT_REGINA_BIFS,unx_uname,             "UNAME" },
+  { EXT_REGINA_BIFS,unx_unixerror,         "UNIXERROR" },
+  { EXT_REGINA_BIFS,arexx_upper,           "UPPER" },
+  { EXT_REGINA_BIFS,rex_userid,            "USERID" },
+  { 0,              std_value,             "VALUE" },
+  { 0,              std_verify,            "VERIFY" },
+  { 0,              std_word,              "WORD" },
+  { 0,              std_wordindex,         "WORDINDEX" },
+  { 0,              std_wordlength,        "WORDLENGTH" },
+  { 0,              std_wordpos,           "WORDPOS" },
+  { 0,              std_words,             "WORDS" },
+  { EXT_AREXX_BIFS, arexx_writech,         "WRITECH" },
+  { EXT_AREXX_BIFS, arexx_writeln,         "WRITELN" },
+  { 0,              std_x2b,               "X2B" },
+  { 0,              std_x2c,               "X2C" },
+  { 0,              std_x2d,               "X2D" },
+  { 0,              std_xrange,            "XRANGE" },
+  { 0,              NULL,                  NULL }
 } ;
 
 static const int num_funcs = sizeof(functions) / (sizeof(functions[0])) - 1 ;
@@ -210,7 +242,7 @@ static const int MonthDays[] = {31,28,31,30,31,30,31,31,30,31,30,31};
 static const int DaysInYear[] = {0,31,59,90,120,151,181,212,243,273,304,334};
 
 static int leapyear(long year) ;
-static int base2date(long basedate,void *conv_tmdata) ;
+static void base2date(long basedate,void *conv_tmdata) ;
 
 #ifdef TRACEMEM
 void mark_listleaked_params( const tsd_t *TSD )
@@ -229,33 +261,17 @@ void mark_listleaked_params( const tsd_t *TSD )
 streng *buildtinfunc( tsd_t *TSD, nodeptr this )
 {
    int low=0, topp=0, mid=0, end=1, up=num_funcs-1, i=0 ;
-   streng *ptr=NULL ;
-   int ext=0 ;
-   void *vptr=NULL ;
+   streng *ptr;
+   struct entry_point *vptr;
    streng *(*func)(tsd_t *,cparamboxptr)=NULL ;
-   streng *upper_name;
 
-   upper_name=Str_upper(Str_dupTSD(this->name));
    /*
     * Look for a function registered in a DLL
     */
-   vptr = loaded_lib_func( TSD, upper_name ) ;
-   if (vptr)
+   vptr = loaded_lib_func( TSD, this->name ) ;
+   if ( vptr )
       func = std_center ; /* e.g. */
 
-   Free_stringTSD( upper_name );
-
-   /*
-    * If no function registered in a DLL, look for one in the
-    * current EXE
-    */
-   if (!func)
-   {
-      ext = external_func( TSD, this->name ) ;
-      if (ext)
-         func = std_center ; /* e.g. */
-   }
-
    /*
     * If no function registered in a DLL or EXE, look for a builtin
     */
@@ -287,25 +303,17 @@ streng *buildtinfunc( tsd_t *TSD, nodeptr this )
          }
          if (!end)
          {
-#if OLD_OPTIONS
-            if (functions[mid].length)
+            /*
+             * Check if the function is an extension. If it is and it matches
+             * an extension specified with the OPTIONS keyword, then allow it.
+             * If the OPTION; STRICT_ANSI is in effect however, this overrides
+             * the extension.
+             */
+            if (functions[mid].compat)
             {
-               int obyte = functions[mid].length/(sizeof(unsigned char)*8) ;
-               int obit = functions[mid].length%(sizeof(unsigned char)*8) ;
-
-               if (!(TSD->currlevel->u.flags[obyte] & (1<<(7-obit))))
-                  func = NULL ;
-               else
-               {
-                  func = functions[mid].function ;
-                  if (TSD->currlevel->u.options.cacheext)
-                     this->u.func = func ;
-               }
-            }
-#else
-            if (functions[mid].length)
-            {
-               if ( ! get_options_flag( TSD->currlevel, functions[mid].length ) )
+               if ( get_options_flag( TSD->currlevel, EXT_STRICT_ANSI ) )
+                  exiterror( ERR_NON_ANSI_FEATURE, 1, functions[mid].funcname );
+               if ( ! get_options_flag( TSD->currlevel, functions[mid].compat ) )
                   func = NULL ;
                else
                {
@@ -314,7 +322,6 @@ streng *buildtinfunc( tsd_t *TSD, nodeptr this )
                      this->u.func = func ;
                }
             }
-#endif
             else
                this->u.func = func = functions[mid].function ;
          }
@@ -332,10 +339,8 @@ streng *buildtinfunc( tsd_t *TSD, nodeptr this )
                      */
 
       TSD->bif_first = initplist( TSD, this ) ;
-      if (ext)
-         ptr = do_an_external_exe( TSD, this->name, TSD->bif_first, 0, (char) this->called) ;
-      else if (vptr)
-         ptr = do_an_external_dll( TSD, vptr, TSD->bif_first, (char) this->called ) ;
+      if (vptr)
+         ptr = call_known_external( TSD, vptr, TSD->bif_first, (char) this->o.called ) ;
       else
          ptr = (*func)(TSD, TSD->bif_first /* ->next */ ) ;
 
@@ -357,22 +362,19 @@ streng *buildtinfunc( tsd_t *TSD, nodeptr this )
 
 
          TSD->bif_first = initplist( TSD, this ) ;
-         ptr = do_an_external_exe( TSD, this->name, TSD->bif_first, 1, (char) this->called ) ;
+         ptr = call_unknown_external( TSD, this->name, TSD->bif_first, (char) this->o.called ) ;
          deallocplink( TSD, TSD->bif_first ) ;
          TSD->bif_first = NULL ;
       }
       else
-         return NOFUNC ;
+         ptr = NOFUNC;
    }
-   /* add return to keep compiler happy. won't ever reach here */
-   return NULL;
+   return ptr;
 }
 
 paramboxptr initplist( tsd_t *TSD, cnodeptr this )
 {
-   paramboxptr first=NULL, new=NULL, currnt=NULL ;
-   streng *pptr=NULL ;
-   streng *junk=NULL ;
+   paramboxptr first,new,currnt;
 
    first = currnt = NULL ;
    for (this=this->p[0]; this; this=this->p[1])
@@ -396,97 +398,92 @@ paramboxptr initplist( tsd_t *TSD, cnodeptr this )
       if (this->type==X_CEXPRLIST && TSD->trace_stat!='I')
       {
          if (this->u.strng)
-            pptr = this->u.strng ;
+            currnt->value = this->u.strng ;
          else
-            pptr = NULL ;
+            currnt->value  = NULL ;
 
          currnt->dealloc = 0 ;
       }
+      else if ( !this->p[0] )
+      {
+         currnt->dealloc = 1;
+         currnt->value = NULL;
+      }
       else
       {
-         currnt->dealloc = 1 ;
-         if (this->p[0])
-            pptr = evaluate( TSD, this->p[0], &junk ) ;
-         else
-            pptr = NULL ;
-
-         if ( get_options_flag( TSD->currlevel, EXT_PGB_PATCH1 ) ) /* pgb */
-         {
-            /*
-            * This code "removed" 16/04/99 byMH, based on suggestion from
-            * Patrick McPhee.  The case that FGC states as a potential
-            * error, does not cause a problem by removing this code.
-            */
-           /*
-            * Reinstated by FGC 25/11/99 to fix bug 35
-            */
-           /*
-            * FGC: HELP! I really don't know if this will work. The problem
-            * is:
-            * a) is pptr a "new" tree with "new" values ? In this case the
-            *    Str_dupTSD() is wrong. What happens in this case if we don't
-            *    delete it later (dealloc=0) ?
-            * b) junk SEEMS to me as the same as pptr if newly allocated
-            * (see expr.c/evaluate). Why, in this case does it crash ?
-            */
-            if (!junk)
-            {
-               if ( pptr != NULL )
-                  pptr = Str_dupTSD( pptr );
-            }
-            /*
-             * Currently only used in dealloc_plink() some lines below.
-             * I think:
-             *  !junk -->no newly allocated
-             *  values --> not allowed to delete them
-             * BUT, WHAT HAPPENS IN CASE OF junk==NULL? SHALL WE Str_dupTSD pptr ?
-             * THERE ARE GOOD REASONS TO DO IT.  TRY THIS FIRST IN CASE OF
-             * STRANGE RESULTS/CRASHES WITH ARGUMENTS.
-             */
-            else
-            {
-            /*
-             * MH - 3-May-2000
-             * added:
-                  if ( pptr )
-                     currnt->dealloc = 1 ;
-                  else
-             * to remove memory leak as reported by PJM - bug 20000502-55023
-             * seems to be stable
-             */
-               if ( pptr )
-                  currnt->dealloc = 1 ;
-               else
-                  currnt->dealloc = 0 ;
-            }
-         }
-         else
-         {
-            /*
-             * FGC: Following code will cause a crash if a variable x is
-             * passed to a function which uses both arg(1) and x: arg(1)
-             * is a reference to x. Modifying x causes a deletion of arg(1)
-             */
-            if (!junk)
-   /*            pptr = Str_dupTSD( pptr ) ; */
-               currnt->dealloc = 0 ;
-         }
-
+         /*
+          * This fixes bug 590589 and others.
+          * Always force a fresh new return value of evaluate.
+          * Imagine this code, it will produce a crash otherwise:
+          * call func x
+          * return
+          * func:
+          *    x = "new" || "value"
+          *    say arg(1)
+          */
+         currnt->dealloc = 1;
+         currnt->value = evaluate( TSD, this->p[0], NULL );
       }
-
-      currnt->value = pptr ;
    }
 #ifdef TRACEMEM
    TSD->listleaked_params = first ;
 #endif
-   currnt->next = NULL ;
+   if ( currnt )
+      currnt->next = NULL ;
    return first ;
 }
 
 
+paramboxptr initargs( tsd_t *TSD, int argc, const int *lengths,
+                      const char **strings )
+{
+   paramboxptr first,new,currnt;
+   int i;
+
+   first = currnt = NULL;
+   for ( i = 0; i < argc; i++ )
+   {
+      if ( TSD->par_stack )
+      {
+         new = TSD->par_stack;
+         TSD->par_stack = new->next;
+      }
+      else
+         new = MallocTSD( sizeof( parambox ) );
+
+      if ( !first )
+         first = currnt = new;
+      else
+      {
+         currnt->next = new;
+         currnt = new;
+      }
+
+      if ( lengths[i] == RX_NO_STRING )
+      {
+         currnt->dealloc = 1;
+         currnt->value = NULL;
+      }
+      else
+      {
+         currnt->value = Str_ncreTSD( strings[i], lengths[i] );
+         currnt->dealloc = 1;
+      }
+   }
+
+#ifdef TRACEMEM
+   TSD->listleaked_params = first;
+#endif
+
+   if ( currnt )
+      currnt->next = NULL;
+   return first;
+}
+
+
 void deallocplink( tsd_t *TSD, paramboxptr first )
 {
-   paramboxptr this=NULL ;
+   paramboxptr this;
 
    for (;first;)
    {
@@ -556,22 +553,47 @@ int atozpos( tsd_t *TSD, const streng *text, const char *bif, int argnum )
 }
 
 
-char getoptionchar( tsd_t *TSD, const streng *text, const char* bif, int argnum, const char *choices )
+char getoptionchar( tsd_t *TSD, const streng *text, const char* bif, int argnum, const char *ansi_choices, const char *regina_choices )
 {
    char ch=0 ;
    const char *ptr = NULL;
+   char tmp[50];
 
    if (text->len == 0)
       exiterror( ERR_INCORRECT_CALL, 21, bif, argnum )  ;
 
-   ch = (char) toupper( text->value[0] ) ;
-
-   for ( ptr = choices; *ptr; ptr++ )
+   ch = (char) rx_toupper( text->value[0] ) ;
+   /*
+    * If the option supplied is ANSI, then return when we find it.
+    */
+   for ( ptr = ansi_choices; *ptr; ptr++ )
    {
       if ( *ptr == ch )
          return ch ;
    }
-   exiterror( ERR_INCORRECT_CALL, 28, bif, argnum, choices, tmpstr_of( TSD, text ) );
+   /*
+    * If the option supplied is a Regina extension, and we are NOT running in
+    * ANSI mode, then return when we find it.
+    */
+   for ( ptr = regina_choices; *ptr; ptr++ )
+   {
+      if ( *ptr == ch )
+      {
+         if ( get_options_flag( TSD->currlevel, EXT_STRICT_ANSI ) )
+            exiterror( ERR_NON_ANSI_FEATURE, 3, bif, argnum, ansi_choices, tmpstr_of( TSD, text ) );
+         else
+            return ch ;
+      }
+   }
+
+   if ( get_options_flag( TSD->currlevel, EXT_STRICT_ANSI ) )
+      exiterror( ERR_INCORRECT_CALL, 28, bif, argnum, ansi_choices, tmpstr_of( TSD, text ) );
+   else
+   {
+      strcpy( tmp, ansi_choices );
+      strcat( tmp, regina_choices );
+      exiterror( ERR_INCORRECT_CALL, 28, bif, argnum, tmp, tmpstr_of( TSD, text ) );
+   }
    return 0 ;
 }
 
@@ -662,7 +684,7 @@ const streng *param( cparamboxptr ptr, int num )
    int i=0 ;
    for (i=1;i<num;i++,ptr=ptr->next)
       if (!ptr)
-         exiterror( ERR_INTERPRETER_FAILURE, 0 )  ;
+         exiterror( ERR_INTERPRETER_FAILURE, 1, __FILE__, __LINE__, "" )  ;
 
    return ((ptr)&&(ptr->value)) ? ptr->value : NULL ;
 }
@@ -675,32 +697,46 @@ const streng *param( cparamboxptr ptr, int num )
  * into a struct tm (individual values for year, month, day, year_days and
  * base days).
  */
-int convert_date(const streng *suppdate, char suppformat, struct tm *indate)
+int convert_date(tsd_t *TSD, const streng *suppdate, char suppformat, struct tm *indate)
 {
    int rc=0,i=0,off=0;
    long num1=0,num2=0,num3=0;
    char buf[20];
-   const char *ptr=suppdate->value;
+   char *ptr=(char*)suppdate->value;
+   struct tm *tmpTime;
 
+   indate->tm_sec = indate->tm_min = indate->tm_hour = 0;
    switch(suppformat)
    {
       case 'B':
-         memcpy(buf,ptr,suppdate->len);
-         buf[suppdate->len] = '\0';
-         if ((num1 = atol(buf)) == 0)
-            for (i=0;i<suppdate->len;i++)
-               if (buf[i] != '0')
-                  return(1);
-         base2date(num1,indate);
-         break;
       case 'D':
+         if (suppdate->len > 19)
+            return(1);
+         memcpy(buf,ptr,suppdate->len);
+         buf[suppdate->len] = '\0';
+         if ((num1 = atol(buf)) == 0)
+         {
+            for (i=0;i<suppdate->len;i++)
+            {
+               if (buf[i] != '0')
+                  return(1);
+            }
+         }
+         if ( suppformat == 'B' )
+            base2date(num1,indate);
+         else
+            base2date(num1+basedays(indate->tm_year)-1,indate);
+         break;
+      case 'I':
+         if (suppdate->len > 19)
+            return(1);
          memcpy(buf,ptr,suppdate->len);
          buf[suppdate->len] = '\0';
          if ((num1 = atol(buf)) == 0)
             for (i=0;i<suppdate->len;i++)
-               if (buf[i] != '0')
-                  return(1);
-         base2date(num1+basedays(indate->tm_year)-1,indate);
+         if (buf[i] != '0')
+            return(1);
+         base2date(num1+basedays(1978)-1,indate);
          break;
       case 'E':
       case 'O':
@@ -711,17 +747,17 @@ int convert_date(const streng *suppdate, char suppformat, struct tm *indate)
               return( 1 );
          memcpy(buf,ptr,2);
          buf[2] = '\0';
-         if ( !isdigit( buf[0] ) || !isdigit( buf[1] ) )
+         if ( !rx_isdigit( buf[0] ) || !rx_isdigit( buf[1] ) )
             return( 1 );
          num1 = atol( buf );
          memcpy(buf,(ptr+3),2);
          buf[2] = '\0';
-         if ( !isdigit( buf[0] ) || !isdigit( buf[1] ) )
+         if ( !rx_isdigit( buf[0] ) || !rx_isdigit( buf[1] ) )
             return( 1 );
          num2 = atol( buf );
          memcpy(buf,(ptr+6),2);
          buf[2] = '\0';
-         if ( !isdigit( buf[0] ) || !isdigit( buf[1] ) )
+         if ( !rx_isdigit( buf[0] ) || !rx_isdigit( buf[1] ) )
             return( 1 );
          num3 = atol( buf );
          switch(suppformat)
@@ -803,6 +839,13 @@ int convert_date(const streng *suppdate, char suppformat, struct tm *indate)
          indate->tm_mon = num2-1;
          indate->tm_year = num1;
          break;
+      case 'T':
+         num1 = streng_to_int( TSD, suppdate, &rc );
+         if ( rc ) return 1;
+         tmpTime = localtime( (time_t *)&num1 );
+         memcpy( indate, tmpTime, sizeof(struct tm) );
+         indate->tm_year += 1900;
+         break;
       default:
          /* should not get here */
          break;
@@ -845,7 +888,7 @@ static int leapyear(long year)
  * Converts a number (representing the number of days since 01 Jan 0001)
  * to a struct tm value (individual fields for year, month, day and days in year)
  */
-static int base2date(long basedate,void *conv_tmdata)
+static void base2date(long basedate,void *conv_tmdata)
 {
    struct tm *outdate=(struct tm *)conv_tmdata;
    int i=0;
@@ -875,7 +918,7 @@ static int base2date(long basedate,void *conv_tmdata)
    outdate->tm_mday = day;
    outdate->tm_yday = yeardays;
    outdate->tm_wday = ((basedate+8) % 7);
-   return(0);
+   return;
 }
 
 /*
@@ -884,32 +927,41 @@ static int base2date(long basedate,void *conv_tmdata)
  */
 int convert_time( const tsd_t *TSD, const streng *supptime, char suppformat, struct tm *intime, time_t *unow)
 {
-   int rc=0;
+   int rc=0,offset;
    long num1=0,num2=0,num3=0,num4=0;
    char buf[20];
-   const char *ptr=supptime->value;
+   char *ptr=(char*)supptime->value;
+   struct tm *tmpTime;
 
    switch(suppformat)
    {
       case 'C':
-         if (*(ptr+2) != ':')
+         /*
+          * Format of time can be "3:45pm", or "11:45pm"; ie hour can be 1 or
+          * two digits. Use of "offset" below fixes bug 742725
+          */
+         if (*(ptr+2) == ':')
+            offset = 1;
+         else if (*(ptr+1) == ':')
+            offset = 0;
+         else
             return(1);
-         if (memcmp("am",ptr+5,2) != 0 && memcmp("pm",ptr+5,2) != 0)
+         if (memcmp("am",ptr+4+offset,2) != 0 && memcmp("pm",ptr+4+offset,2) != 0)
             return(1);
-         memcpy(buf,ptr,2);
-         buf[2] = '\0';
+         memcpy(buf,ptr,1+offset);
+         buf[1+offset] = '\0';
          if ((num1 = atol(buf)) == 0 && strcmp("00",buf) != 0)
             return(1);
          if (num1 > 12)
             return(1);
-         memcpy(buf,ptr+3,2);
+         memcpy(buf,ptr+2+offset,2);
          buf[2] = '\0';
          if ((num2 = atol(buf)) == 0 && strcmp("00",buf) != 0)
             return(1);
          if (num2 > 59)
             return(1);
          intime->tm_sec = 0;
-         if (memcmp("am",ptr+5,2)==0)
+         if (memcmp("am",ptr+4+offset,2)==0)
          {
             if (num1 == 12)
                intime->tm_hour = 0;
@@ -935,6 +987,8 @@ int convert_time( const tsd_t *TSD, const streng *supptime, char suppformat, str
           */
          num1 = streng_to_int( TSD, supptime, &rc );
          if ( rc ) return 1;
+         if ( num1 < 0 )
+            return(1);
          switch(suppformat)
          {
             case 'H':
@@ -952,7 +1006,7 @@ int convert_time( const tsd_t *TSD, const streng *supptime, char suppformat, str
                intime->tm_sec = (num1 % 3600) % 60;
                break;
          }
-         if (intime->tm_sec > 59 || intime->tm_hour > 23 || intime->tm_min > 59)
+         if ( intime->tm_sec > 59 || intime->tm_hour > 23 || intime->tm_min > 59 )
             return(1);
          *unow = 0;
          break;
@@ -1002,6 +1056,13 @@ int convert_time( const tsd_t *TSD, const streng *supptime, char suppformat, str
             return(1);
          *unow = num4;
          break;
+      case 'T':
+         num1 = streng_to_int( TSD, supptime, &rc );
+         if ( rc ) return 1;
+         tmpTime = localtime( (time_t *)&num1 );
+         memcpy( intime, tmpTime, sizeof(struct tm) );
+         *unow = 0;
+         break;
       default:
          /* should not get here */
          break;
@@ -1009,3 +1070,31 @@ int convert_time( const tsd_t *TSD, const streng *supptime, char suppformat, str
 
    return(rc);
 }
+/*
+ * The following functions are wrappers for BIFs that are syntactically different
+ * depending on the OPTONS used.
+ */
+
+static streng *conflict_close( tsd_t *TSD, cparamboxptr parms )
+{
+   if ( get_options_flag( TSD->currlevel, EXT_AREXX_SEMANTICS ) )
+      return( arexx_close( TSD, parms ) );
+   else
+      return( unx_close( TSD, parms ) );
+}
+
+static streng *conflict_eof( tsd_t *TSD, cparamboxptr parms )
+{
+   if ( get_options_flag( TSD->currlevel, EXT_AREXX_SEMANTICS ) )
+      return( arexx_eof( TSD, parms ) );
+   else
+      return( unx_eof( TSD, parms ) );
+}
+
+static streng *conflict_open( tsd_t *TSD, cparamboxptr parms )
+{
+   if ( get_options_flag( TSD->currlevel, EXT_AREXX_SEMANTICS ) )
+      return( arexx_open( TSD, parms ) );
+   else
+      return( unx_open( TSD, parms ) );
+}
