@@ -23,7 +23,7 @@
  * that we must create a pointerless list which is included
  * in the tinned tree.
  *
- * This structure is called ttree, have a look at types.h.
+ * This structure is called ttree, have a look at regina_t.h.
  *
  * DEFAULT_TTREE_ELEMS defines the normal number of elements in the ttree.
  * The more elements the better is the performance in searching. The less
@@ -127,7 +127,7 @@ nodeptr FreshNode(void)
  */
 void RejectNode(nodeptr NoLongerUsed)
 {
-   unsigned idx;
+   unsigned long idx;
 
    assert(CurrentT != NULL);
    /* CurrentT == NULL can't happen, since CurrentT is only set within the
@@ -202,26 +202,27 @@ static void DestroyNode(const tsd_t *TSD, nodeptr p)
 
    type = p->type ;
    if (type == X_CON_SYMBOL || type == X_STRING)
+   {
       if (p->u.number)
       {
          FreeTSD( p->u.number->num ) ;
          FreeTSD( p->u.number ) ;
       }
-
+   }
    if (type==X_SIM_SYMBOL || type==X_STEM_SYMBOL || type==X_HEAD_SYMBOL ||
        type==X_CTAIL_SYMBOL || type==X_VTAIL_SYMBOL )
+   {
       if (p->u.varbx)
       {
-#ifdef DEBUG
          detach( TSD, p->u.varbx ) ;
-#else
-         detach( p->u.varbx ) ;
-#endif
       }
+   }
 
    if (type == X_CEXPRLIST)
+   {
       if (p->u.strng)
          Free_stringTSD( p->u.strng ) ;
+   }
 }
 
 /* DestroyInternalParsingTree frees all allocated memory used by a parsing
@@ -233,7 +234,7 @@ void DestroyInternalParsingTree(const tsd_t *TSD, internal_parser_type *ipt)
    otree *or, *oh;
    lineboxptr lr, lh;
    labelboxptr ar, ah;
-   unsigned i;
+   unsigned long i;
 
    if (!ipt)
       return;
@@ -243,7 +244,8 @@ void DestroyInternalParsingTree(const tsd_t *TSD, internal_parser_type *ipt)
    {
       tr = ipt->nodes;
 
-      while (tr) {
+      while (tr)
+      {
          for (i = 0; i < tr->num; i++)
             DestroyNode(TSD, tr->elems + i);
          th = tr->next;
@@ -261,7 +263,8 @@ void DestroyInternalParsingTree(const tsd_t *TSD, internal_parser_type *ipt)
    {
       lr = ipt->first_source_line;
 
-      while (lr) {
+      while (lr)
+      {
          lh = lr->next;
          Free_stringTSD(lr->line);
          FreeTSD(lr);
@@ -276,7 +279,8 @@ void DestroyInternalParsingTree(const tsd_t *TSD, internal_parser_type *ipt)
    {
       ar = ipt->first_label;
 
-      while (ar) {
+      while (ar)
+      {
          ah = ar->next;
          FreeTSD(ar);
          ar = ah;
@@ -298,7 +302,8 @@ void DestroyInternalParsingTree(const tsd_t *TSD, internal_parser_type *ipt)
    {
       or = ipt->srclines;
 
-      while (or) {
+      while (or)
+      {
          oh = or->next;
          FreeTSD(or->elems);
          FreeTSD(or);
@@ -326,7 +331,7 @@ internal_parser_type ExpandTinnedTree(const tsd_t *TSD,
                                       unsigned long incore_source_length)
 {
    internal_parser_type ipt;
-   unsigned i,j;
+   unsigned long i,j;
    const extstring *es;
    const offsrcline *lastsrcline;
    nodeptr this;
@@ -394,64 +399,87 @@ internal_parser_type ExpandTinnedTree(const tsd_t *TSD,
       this = ipt.nodes->elems + i;
       if (this->name)
       {
-         es = (extstring *) ((char *) ept + (unsigned) this->name);
+         es = (extstring *) ((char *) ept + (unsigned long) this->name);
          this->name = Str_makeTSD(es->length);
          this->name->len = es->length;
          memcpy(this->name->value,
                 es + 1 /* position of string content */,
                 es->length);
       }
-      if (this->u.strng)
+
+      /*
+       * Do things the parsing step would have do. Simple values in this->u
+       * are copied already.
+       */
+
+      /*
+       * See also several places in this file and in debug.c where this
+       * switch list must be changed. Seek for X_CEXPRLIST.
+       */
+      switch ( this->type )
       {
-         es = (extstring *) ((char *) ept + (unsigned) this->u.strng);
-         this->u.strng = Str_makeTSD(es->length);
-         this->u.strng->len = es->length;
-         memcpy(this->u.strng->value,
-                es + 1 /* position of string content */,
-                es->length);
+         case X_CEXPRLIST:
+            if ( this->u.strng )
+            {
+               es = (extstring *) ((char *) ept + (unsigned long) this->u.strng);
+               this->u.strng = Str_makeTSD( es->length );
+               this->u.strng->len = es->length;
+               memcpy( this->u.strng->value,
+                       es + 1 /* position of string content */,
+                       es->length);
+            }
+            break;
+
+         case X_LABEL:
+            newlabel(TSD, &ipt, this);
+            break;
+
+         default:
+            break;
       }
 
-      if (this->type == X_LABEL)
-         newlabel(TSD, &ipt, this);
-
-      if (this->next == (nodeptr) (unsigned) -1)
+      if (this->next == (nodeptr) (unsigned long) -1)
          this->next = NULL;
       else
-         this->next = ipt.nodes->elems + (unsigned) this->next;
+         this->next = ipt.nodes->elems + (unsigned long) this->next;
       for (j = 0;j < sizeof(this->p) / sizeof(this->p[0]);j++)
-         if (this->p[j] == (nodeptr) (unsigned) -1)
+      {
+         if (this->p[j] == (nodeptr) (unsigned long) -1)
             this->p[j] = NULL;
          else
-            this->p[j] = ipt.nodes->elems + (unsigned) this->p[j];
+            this->p[j] = ipt.nodes->elems + (unsigned long) this->p[j];
+      }
    }
+   size = size; /* keep compiler happy */
 
    return(ipt);
 }
 
 /* We must take care of the alignment of structure. We may get a SIGBUS in
  * the following if we don't do it. We assume that an alignment for an
- * unsigned is sufficient for all types including structures. We also
+ * unsigned long is sufficient for all types including structures. We also
  * assume a power of two for an unsigned's size.
  */
-#define USIZ sizeof(unsigned)
+#define USIZ sizeof(unsigned long)
 #define USIZ_1 (USIZ-1)
 /* Wastes one byte in average but is much faster */
 #define StringSize(s) (((sizeof(extstring)+s->len)|USIZ_1)+1)
 
-static unsigned ComputeExternalSize(const internal_parser_type *ipt,
-                                    unsigned *SourceLines,
-                                    unsigned *Nodes)
+static unsigned long ComputeExternalSize(const internal_parser_type *ipt,
+                                         unsigned long *SourceLines,
+                                         unsigned long *Nodes)
 {
    otree *otp;
    ttree *ttp;
    nodeptr np;
-   unsigned size = sizeof(external_parser_type);
-   unsigned i, elems, bufchars;
+   unsigned long size = sizeof(external_parser_type);
+   unsigned long i, elems, bufchars;
 
    /* sourceline table */
    elems = 0;
    otp = ipt->srclines;
-   while (otp != NULL) {
+   while (otp != NULL)
+   {
       elems += otp->sum;
       otp = otp->next;
    }
@@ -461,7 +489,8 @@ static unsigned ComputeExternalSize(const internal_parser_type *ipt,
    /* nodetable */
    elems = bufchars = 0;
    ttp = ipt->nodes;
-   while (ttp) {
+   while (ttp)
+   {
       for (i = 0;i < ttp->num;i++)
       {
          elems++;
@@ -469,9 +498,25 @@ static unsigned ComputeExternalSize(const internal_parser_type *ipt,
          if (np->name)
             bufchars += StringSize(np->name);
 
-         if ((np->type == X_CEXPRLIST) && np->u.strng)
-           /* should be save, since it isn't computed after the parsing step.*/
-            bufchars += StringSize(np->u.strng);
+         /*
+          * Add all sizes of strings that have been generated at the parsing
+          * step.
+          */
+
+         /*
+          * See also several places in this file and in debug.c where this
+          * switch list must be changed. Seek for X_CEXPRLIST.
+          */
+         switch ( np->type )
+         {
+            case X_CEXPRLIST:
+               if ( np->u.strng )
+                  bufchars += StringSize( np->u.strng );
+            break;
+
+            default:
+               break;
+         }
       }
       ttp = ttp->next;
    }
@@ -487,9 +532,10 @@ static unsigned ComputeExternalSize(const internal_parser_type *ipt,
  * consecutively.
  * The index just beyond the last copied byte is returned.
  */
-static unsigned FillStrings(char *base, unsigned start, const otree *otp)
+static unsigned long FillStrings(char *base, unsigned long start, const otree *otp)
 {
-   while (otp != NULL) {
+   while (otp != NULL) 
+   {
       memcpy(base + start, otp->elems, otp->num * sizeof(offsrcline));
       start += otp->num * sizeof(offsrcline);
       otp = otp->next;
@@ -498,20 +544,21 @@ static unsigned FillStrings(char *base, unsigned start, const otree *otp)
 }
 
 /* FillTree copies all treenodes of the ttree to base+buf in a relocatable
- * manner. Look at ExpandTinnedTree() or types.h for a description.
+ * manner. Look at ExpandTinnedTree() or regina_t.h for a description.
  * Each treenode is copied to the table and the containing strings are copied
  * as extstrings to base+start which is incremented.
  * The table must be large enough.
  * The index just beyond the last copied character is returned.
  */
-static unsigned FillTree(treenode *table, char *base, unsigned start,
+static unsigned long FillTree(treenode *table, char *base, unsigned long start,
                          const ttree *ttp)
 {
    cnodeptr np;
-   unsigned i,j;
+   unsigned long i,j;
    extstring *e;
 
-   while (ttp) {
+   while (ttp) 
+   {
       for (i = 0;i < ttp->num;i++)
       {
          np = (cnodeptr) (ttp->elems + i);
@@ -526,27 +573,66 @@ static unsigned FillTree(treenode *table, char *base, unsigned start,
             start += StringSize(np->name);
          }
 
-         if ((np->type == X_CEXPRLIST) && np->u.strng)
+         /*
+          * Remove all "flags" from the target and copy only approved values
+          * the parser computes already.
+          */
+         memset( &table->u, 0, sizeof( table->u ) );
+
+         /*
+          * See also several places in this file and in debug.c where this
+          * switch list must be changed. Seek for X_CEXPRLIST.
+          */
+         switch ( np->type )
          {
-            table->u.strng = (streng *) start;
-            e = (extstring *) (base + start);
-            e->length = np->u.strng->len;
-            memcpy(e + 1, np->u.strng->value, e->length);
-            start += StringSize(np->u.strng);
+            case X_EQUAL:
+            case X_DIFF:
+            case X_GT:
+            case X_GTE:
+            case X_LT:
+            case X_LTE:
+               table->u.flags = np->u.flags;
+               break;
+
+            case X_ADDR_V:
+               table->u.nonansi = np->u.nonansi;
+               break;
+
+            case X_CEXPRLIST:
+               if ( np->u.strng )
+               {
+                  table->u.strng = (streng *) start;
+                  e = (extstring *) (base + start);
+                  e->length = np->u.strng->len;
+                  memcpy(e + 1, np->u.strng->value, e->length);
+                  start += StringSize(np->u.strng);
+               }
+            break;
+
+            case X_LABEL:
+               table->u.trace_only = np->u.trace_only;
+               break;
+
+            case X_ADDR_WITH:
+               if ( !np->p[0] && !np->p[1] && !np->p[2] )
+                  table->u.of = np->u.of;
+               break;
+
+            default:
+               break;
          }
-         else
-            table->u.strng = 0; /* Be sure not to run into troubles */
 
          if (table->next == NULL)
-            table->next = (nodeptr) (unsigned) -1;
+            table->next = (nodeptr) (unsigned long) -1;
          else
             table->next = (nodeptr) np->next->nodeindex;
          for (j = 0;j < sizeof(np->p) / sizeof(np->p[0]);j++)
+         {
             if (table->p[j] == NULL)
-               table->p[j] = (nodeptr) (unsigned) -1;
+               table->p[j] = (nodeptr) (unsigned long) -1;
             else
                table->p[j] = (nodeptr) np->p[j]->nodeindex;
-
+         }
          table++;
       }
       ttp = ttp->next;
@@ -559,14 +645,14 @@ static unsigned FillTree(treenode *table, char *base, unsigned start,
  * is allocated by one call to IfcAllocateMemory. The returned value shall
  * be used as an instore macro for RexxStart.
  * *length is set to the allocated size of the memory block on return.
- * ExpandedTinnedTree can expand the returned value and IaValidTin checks it.
+ * ExpandedTinnedTree can expand the returned value and IsValidTin checks it.
  */
 external_parser_type *TinTree(const tsd_t *TSD,
                               const internal_parser_type *ipt,
                               unsigned long *length)
 {
    external_parser_type *retval;
-   unsigned srclines, nodecount, len;
+   unsigned long srclines, nodecount, len;
 
    *length = ComputeExternalSize(ipt, &srclines, &nodecount);
 
@@ -589,7 +675,7 @@ external_parser_type *TinTree(const tsd_t *TSD,
    retval->arch_detector.s.two = 2;
    retval->arch_detector.s.ptr3 = (void *)3;
    retval->arch_detector.s.ptr4 = (void *)4;
-   retval->OverallSize = (unsigned) *length;
+   retval->OverallSize = (unsigned long) *length;
    retval->NumberOfSourceLines = srclines;
    retval->version = INSTORE_VERSION;
    retval->NumberOfTreeElements = nodecount;
@@ -611,6 +697,7 @@ external_parser_type *TinTree(const tsd_t *TSD,
    assert((unsigned long) len + sizeof(retval->Magic) == *length);
 
    /* DEBUGGING: return NULL if you don't want tinned trees */
+   TSD = TSD; /* keep compiler happy */
    return(retval);
 }
 
@@ -620,7 +707,7 @@ external_parser_type *TinTree(const tsd_t *TSD,
 int IsValidTin(const external_parser_type *ept, unsigned long eptlen)
 {
    char Magic[sizeof(((external_parser_type *)0)->Magic)];
-   unsigned len;
+   unsigned long len;
 
    /* Some paranoia tests first: */
    if ((ept == NULL) || (eptlen < sizeof(external_parser_type)))
